@@ -21,6 +21,7 @@ use File::Path qw(rmtree);
 use File::Spec;
 use List::Util qw(max);
 use LaTeXML::Common::Config;
+use LaTeXML::Common::Error;
 use LaTeXML::Core;
 use LaTeXML::Util::Pack;
 use LaTeXML::Util::Pathname;
@@ -29,7 +30,7 @@ use LaTeXML::Util::ObjectDB;
 use LaTeXML::Post::Scan;
 use vars qw($VERSION);
 # This is the main version of LaTeXML being claimed.
-use version; our $VERSION = version->declare("0.8.3");
+use version; our $VERSION = version->declare("0.8.6");
 use LaTeXML::Version;
 # Derived, more informative version numbers
 our $FULLVERSION = "LaTeXML version $LaTeXML::VERSION"
@@ -44,7 +45,6 @@ our $LOG_STACK = 0;
 
 # Switching to white-listing options that are important for new_latexml:
 our @COMPARABLE = qw(preload paths verbosity strict comments inputencoding includestyles documentid mathparse);
-our %DAEMON_DB = ();
 
 sub new {
   my ($class, $config) = @_;
@@ -54,24 +54,23 @@ sub new {
     latexml => undef }, $class;
   # Special check if the debug directive is on, just to neutralize the bind_log
   my $debug_directives = $$self{opts}->{debug};
-  $LaTeXML::DEBUG = 1 if (ref $debug_directives eq 'ARRAY') && (grep { /latexml/i } @$debug_directives);
-  $self->bind_log;
-  my $rv = eval { $config->check; };
-  $$self{log} .= $self->flush_log;
+  $LaTeXML::DEBUG{latexml} = 1 if (ref $debug_directives eq 'ARRAY') && (grep { /latexml/i } @$debug_directives);
+  eval { $config->check; };
+  if ($@) {
+    $self->bind_log;
+    NoteLog($@);
+    $$self{log} .= $self->flush_log; }
+
   return $self; }
 
 sub prepare_session {
   my ($self, $config) = @_;
-  # TODO: The defaults feature was never used, do we really want it??
-  #0. Ensure all default keys are present:
-  # (always, as users can specify partial options that build on the defaults)
-  #foreach (keys %{$$self{defaults}}) {
-  #  $config->{$_} = $$self{defaults}->{$_} unless exists $config->{$_};
-  #}
   # 1. Ensure option "sanity"
-  $self->bind_log;
-  my $rv = eval { $config->check; };
-  $$self{log} .= $self->flush_log;
+  eval { $config->check; };
+  if ($@) {
+    $self->bind_log;
+    NoteLog($@);
+    $$self{log} .= $self->flush_log; }
 
   my $opts                 = $config->options;
   my $opts_comparable      = { map { $_ => $$opts{$_} } @COMPARABLE };
@@ -91,7 +90,10 @@ sub prepare_session {
 
 sub initialize_session {
   my ($self) = @_;
-  $$self{runtime} = {};
+  $$self{runtime} = {} unless $$self{runtime};
+  # TTL = time-to-live, starting with the allotted timeout (or 0 for unlimited)
+  my $runtime = $$self{runtime};
+  $$runtime{TTL} = $$self{opts}{timeout} unless defined $$runtime{TTL};
   $self->bind_log;
   # Empty the package namespace
   foreach my $subname (keys %LaTeXML::Package::Pool::) {
@@ -99,22 +101,21 @@ sub initialize_session {
   }
 
   my $latexml;
-  my $init_eval_return = eval {
+  eval {
     # Prepare LaTeXML object
     local $SIG{'ALRM'} = sub { die "Fatal:conversion:init Failed to initialize LaTeXML state\n" };
-    alarm($$self{opts}{timeout});
+    alarm($$runtime{TTL});
 
     $latexml = new_latexml($$self{opts});
 
-    alarm(0);
+    $$runtime{TTL} = alarm(0);
     1;
   };
   ## NOTE: This will give double errors, if latexml has already handled it!
   $$latexml{state}->noteStatus('fatal') if $latexml && $@;    # Fatal Error?
-  local $@ = 'Fatal:conversion:unknown Session initialization failed! (Unknown reason)' if ((!$init_eval_return) && (!$@));
   if ($@) {                                                   #Fatal occured!
-    print STDERR "$@\n";
-    print STDERR "\nInitialization complete: " . $latexml->getStatusMessage . ". Aborting.\n" if defined $latexml;
+    Debug($@);
+    Note("Initialization complete: " . $latexml->getStatusMessage . ". Aborting.") if defined $latexml;
     # Close and restore STDERR to original condition.
     $$self{log} .= $self->flush_log;
     $$self{ready} = 0;
@@ -123,7 +124,7 @@ sub initialize_session {
     # Demand errorless initialization
     my $init_status = $latexml->getStatusMessage;
     if ($init_status =~ /error/i) {
-      print STDERR "\nInitialization complete: " . $init_status . ". Aborting.\n";
+      Note("Initialization complete: " . $init_status . ". Aborting.");
       $$self{log} .= $self->flush_log;
       $$self{ready} = 0;
       return;
@@ -141,21 +142,21 @@ sub convert {
   my ($self, $source) = @_;
   # 1 Prepare for conversion
   # 1.1 Initialize session if needed:
-  $$self{runtime} = {};
+  # TTL = time-to-live, starting with the allotted timeout
+  #       on each call to convert (or 0 for unlimited)
+  $$self{runtime} = { TTL => $$self{opts}{timeout} };
+  my $runtime = $$self{runtime};
   $self->initialize_session unless $$self{ready};
   if (!$$self{ready}) {    # We can't initialize, return error:
-    return { result => undef, log => $$self{log}, status => "Initialization failed.", status_code => 3 };
-  }
+    return { result => undef, log => $self->flush_log, status => "Initialization failed.", status_code => 3 }; }
 
   $self->bind_log;
   # 1.2 Inform of identity, increase conversion counter
-  my $opts    = $$self{opts};
-  my $runtime = $$self{runtime};
+  my $opts = $$self{opts};
   ($$runtime{status}, $$runtime{status_code}) = (undef, undef);
-  if ($$opts{verbosity} >= 0) {
-    print STDERR "$LaTeXML::IDENTITY\n";
-    print STDERR "invoked as [$0 " . join(' ', @ARGV) . "]\n" if $$opts{verbosity} >= 1;
-    print STDERR "processing started " . localtime() . "\n"; }
+  Note("$LaTeXML::IDENTITY");
+  NoteLog("invoked as [$0 " . join(' ', @ARGV) . "]");
+  Note(($$opts{recursive} ? "recursive " : "") . "processing started " . localtime());
 
   # 1.3 Prepare for What's IN:
   # We use a new temporary variable to avoid confusion with daemon caching
@@ -191,9 +192,9 @@ sub convert {
   if ($$opts{whatsout} =~ /^archive/) {
     $$opts{archive_sitedirectory} = $$opts{sitedirectory};
     $$opts{archive_destination}   = $$opts{destination};
-    my $destination_name = $$opts{destination} ? pathname_name($$opts{destination}) : 'document';
+    my $destination_name  = $$opts{destination} ? pathname_name($$opts{destination}) : 'document';
     my $sandbox_directory = File::Temp->newdir(TMPDIR => 1);
-    my $extension = $$opts{format};
+    my $extension         = $$opts{format};
     $extension =~ s/\d+$//;
     $extension =~ s/^epub|mobi$/xhtml/;
     my $sandbox_destination = "$destination_name.$extension";
@@ -203,58 +204,99 @@ sub convert {
       $$opts{resource_directory} = File::Spec->catdir($sandbox_directory, 'OPS');
       $$opts{destination} = pathname_concat(File::Spec->catdir($sandbox_directory, 'OPS'), $sandbox_destination); }
     else {
-      $$opts{destination} = pathname_concat($sandbox_directory, $sandbox_destination); }
-  }
+      $$opts{destination} = pathname_concat($sandbox_directory, $sandbox_destination); } }
+  # 1.4.1 Since we can allow "virtual" destinations for archives / webservice APIs,
+  #       we postpone the auxiliary resource sanity check (logically of LaTeXML::Config)
+  #       to this time, where we can be certain if a user has run a local job without --dest
+  if ((!$$opts{destination})
+    && ($$opts{dographics} || $$opts{picimages} || grep { $_ eq 'images' or $_ eq 'svg' } @{ $$opts{math_formats} })) {
+    Warn("expected", "options", undef,
+      "must supply --destination to support auxilliary files",
+      "  disabling: --nomathimages --nographicimages --nopictureimages");
+    # default resources is sorta ok: we might not copy, but we'll still have the links/script/etc
+    $$opts{dographics} = 0;
+    $$opts{picimages}  = 0;
+    removeMathFormat($opts, 'images');
+    removeMathFormat($opts, 'svg');
+    maybeAddMathFormat($opts, 'pmml'); }
 
   # 1.5 Prepare a daemon frame
   my $latexml = $$self{latexml};
   $latexml->withState(sub {
       my ($state) = @_;    # Sandbox state
       $$state{status} = {};
+      my $stomach = $$state{stomach};
+      delete $$stomach{rescued_boxes} if $$stomach{rescued_boxes};
       $state->pushDaemonFrame;
-      $state->assignValue('_authlist', $$opts{authlist}, 'global');
+      $state->assignValue('_authlist',      $$opts{authlist}, 'global');
       $state->assignValue('REMOTE_REQUEST', (!$$opts{local}), 'global');
   });
 
   # 2 Beginning Core conversion - digest the source:
   my ($digested, $dom, $serialized) = (undef, undef, undef);
-  my $convert_eval_return = eval {
-    # Should be this, but is overridden by withState.
-    #local $SIG{'ALRM'} = sub { LaTeXML::Common::Error::Fatal('conversion','timeout',
-    # "Conversion timed out after " . $$opts{timeout} . " seconds!\n"); };
-    alarm($$opts{timeout});
+  eval {
+    alarm($$runtime{TTL});
     my $mode = ($$opts{type} eq 'auto') ? 'TeX' : $$opts{type};
     $digested = $latexml->digestFile($source, preamble => $current_preamble,
       postamble    => $current_postamble,
       mode         => $mode,
       noinitialize => 1);
-    # 2.1 Now, convert to DOM and output, if desired.
-    if ($digested) {
-      $latexml->withState(sub {
-          if ($$opts{format} eq 'tex') {
-            $serialized = LaTeXML::Core::Token::UnTeX($digested);
-          } elsif ($$opts{format} eq 'box') {
-            $serialized = ($$opts{verbosity} > 0 ? $digested->stringify : $digested->toString);
-          } else {    # Default is XML
-            $dom = $latexml->convertDocument($digested);
-          }
-      }); }
-    alarm(0);
-    1;
-  };
-  # 2.2 Bookkeeping in case fatal errors occurred
-  ### Note: this cause double counting if LaTeXML has already handled it.
-  ### But leaving it might might miss errors that sneak through (can that happen?)
-  ####  $$latexml{state}->noteStatus('fatal') if $latexml && $@;    # Fatal Error?
-  local $@ = 'Fatal:conversion:unknown TeX to XML conversion failed! (Unknown Reason)' if ((!$convert_eval_return) && (!$@));
+    $$runtime{TTL} = alarm(0); };
   my $eval_report = $@;
+  if (!$digested && $eval_report) {
+    # We can retry finishing digestion if hit a Fatal,
+    # sometimes there are leftover boxes we can accept.
+    eval {
+      alarm($$runtime{TTL});
+      $digested = $latexml->withState(sub {
+          return $latexml->finishDigestion; });
+      $$runtime{TTL} = alarm(0); };
+    $eval_report .= $@ if $@; }
+  # 2.1 Now, convert to DOM and output, if desired.
+  my $core_target = $$opts{format};
+  # Default Core target is XML
+  if ($core_target ne 'tex' and $core_target ne 'box') {
+    $core_target = 'xml'; }
+  if ($digested) {
+    eval {
+      alarm($$runtime{TTL});
+      $latexml->withState(sub {
+          if ($core_target eq 'tex') {
+            $serialized = LaTeXML::Core::Token::UnTeX($digested); }
+          elsif ($core_target eq 'box') {
+            $serialized = ($$opts{verbosity} > 0 ? $digested->stringify : $digested->toString); }
+          elsif ($core_target eq 'xml') {
+            $dom = $latexml->convertDocument($digested); } });
+      $$runtime{TTL} = alarm(0); };
+    $eval_report .= $@ if $@;
+    # Try to rescue the document if e.g. math parsing hit a Fatal error
+    if (!$dom && $@ && $core_target eq 'xml') {
+      $dom = $latexml->withState(sub {
+          my ($state) = @_;
+          my $rescued = $$state{rescued_document};
+          $rescued->finalize() if $rescued;
+          return $rescued; }); } }
   $$runtime{status}      = $latexml->getStatusMessage;
   $$runtime{status_code} = $latexml->getStatusCode;
+  # 2.2 Bookkeeping in case in-eval perl die() deaths occurred
+  if ($eval_report) {
+    $$runtime{status} .= "\n" . $eval_report . "\n";
+    $$runtime{status_code} = 3; }
 
   # End daemon run, by popping frame:
   $latexml->withState(sub {
       my ($state) = @_;    # Remove current state frame
+      ## TODO: This section of option preparations can be factored out as a subroutine if it grows further
+      ##       the general idea is that right before the "pop" of the daemon frame, we have access to all meaningful
+      ##       global state values, and we can preserve the relevant ones for the post-processing stage
+      ## BEGIN POST-PROCESSING-PREP
       $$opts{searchpaths} = $state->lookupValue('SEARCHPATHS'); # save the searchpaths for post-processing
+      if ($state->lookupValue('LEXEMATIZE_MATH')) {  # save potential request for serializing math lexemes
+        $$opts{math_formats} ||= [];
+        push @{ $$opts{math_formats} }, 'lexemes';
+        # recheck need for parallel
+        $$opts{parallelmath} = 1 if (@{ $$opts{math_formats} } > 1); }
+      ## END POST-PROCESSING-PREP
       $state->popDaemonFrame;
   });
   if ($LaTeXML::UNSAFE_FATAL) {
@@ -262,28 +304,7 @@ sub convert {
     $LaTeXML::UNSAFE_FATAL = 0;
     $$self{ready} = 0;
   }
-  if ($eval_report || ($$runtime{status_code} == 3)) {
-    # Terminate immediately on Fatal errors
-    $$runtime{status_code} = 3;
-    print STDERR $eval_report . "\n" if $eval_report;
-    print STDERR "\nConversion complete: " . $$runtime{status} . ".\n";
-    print STDERR "Status:conversion:" . ($$runtime{status_code} || '0') . "\n";
-    # If we just processed an archive, clean up sandbox directory.
-    if ($$opts{whatsin} =~ /^archive/) {
-      rmtree($$opts{sourcedirectory});
-      $$opts{sourcedirectory} = $$opts{archive_sourcedirectory}; }
-
-    # Close and restore STDERR to original condition.
-    my $log = $self->flush_log;
-    $serialized = $dom if ($$opts{format} eq 'dom');
-    $serialized = $dom->toString if ($dom && (!defined $serialized));
-    # Using the Core::Document::serialize_aux, so need an explicit encode into bytes
-    $serialized = Encode::encode('UTF-8', $serialized) if $serialized;
-
-    return { result => $serialized, log => $log, status => $$runtime{status}, status_code => $$runtime{status_code} }; }
-  else {
-    # Standard report, if we're not in a Fatal case
-    print STDERR "\nConversion complete: " . $$runtime{status} . ".\n"; }
+  Note(($$opts{recursive} ? "recursive " : "") . "Conversion complete: " . $$runtime{status});
 
   # 2.3 Clean up and exit if we only wanted the serialization of the core conversion
   if ($serialized) {
@@ -292,8 +313,7 @@ sub convert {
     if ($$opts{whatsin} =~ /^archive/) {
       rmtree($$opts{sourcedirectory});
       $$opts{sourcedirectory} = $$opts{archive_sourcedirectory}; }
-    my $log = $self->flush_log;
-    return { result => $serialized, log => $log, status => $$runtime{status}, status_code => $$runtime{status_code} };
+    return { result => $serialized, log => $self->flush_log, status => $$runtime{status}, status_code => $$runtime{status_code} };
   }
 
   # 3 If desired, post-process
@@ -317,47 +337,81 @@ sub convert {
     if (delete $$opts{placeholder_destination}) {
       delete $$opts{destination}; } }
 
-  # 5 Output
-  # 5.1 Serialize the XML/HTML result (or just return the Perl object, if requested)
+# 5 Output -- if not using Post::Writer, which never considers this serialization logic
+# 5.1 Serialize the XML/HTML result (or just return the Perl object, if requested)
+# GOAL: $serialized must contain a utf8-encoded string at the return.
+# NOTES: This is difficult, because we can be serializing different objects, with different serialization logic
+# 1. Byte strings: LaTeXML's Document::serialize_aux, and XML::LibXML::Document's toString and toStringHTML
+#    which have NOT been encoded into utf-8, so we need an explicit encode before printing/returning
+# 2. Unreliable: the fragment case, which uses XML::LibXML::Element (and hence Node's) toString method,
+#    is sometimes already encoded as UTF-8. In fact, the documentation claims it is by default:
+# https://metacpan.org/pod/distribution/XML-LibXML/lib/XML/LibXML/Node.pod#toString
+# Digging to the bottom of the code, we reach:
+# https://metacpan.org/source/SHLOMIF/XML-LibXML-2.0200/LibXML.xs#L5212
+# which *will* set the utf8 flag iff the "HAVE_UTF8" flag was set on the system which compiled libxml
+# that's pretty terrifying in fact, since we can't rely that libxml returns the same encoding cross-platform for its default behavior!
+# 3. Reliable: Always explicitly request the document encoding to be used in serializing a Node
+#     by passing a second true flag into toString(1,1) to ensure that the encoding is handled explicitly at the libxml2 level
+# 4. Other: returning a DOM object programmatically, or a non-XML representation (archives), has no serialization component, result is returned as-is
   undef $serialized;
-  if ((defined $result) && ref($result) && (ref($result) =~ /^(:?LaTe)?XML/)) {
-    if (($$opts{format} =~ /x(ht)?ml/) || ($$opts{format} eq 'jats')) {
-      $serialized = $result->toString(1);
-      if (ref($result) eq 'LaTeXML::Core::Document') {
-        # NOTE that we are serializing here via LaTeXML's Document::serialize_aux
-        # which has NOT been encoded into bytes, so we need an explicit encode before printing/returning
+  my $ref_result = ref($result) || '';
+  if ($$opts{format} eq 'dom') {    # No serialize needed in DOM output case
+    $serialized = $result; }
+  elsif ($ref_result =~ /^(:?LaTe)?XML/) {
+    if ($$opts{format} =~ /^jats|x(ht)?ml$/) {
+      if ($ref_result =~ /Document$/) {
+        $serialized = $result->toString(1);
         $serialized = Encode::encode('UTF-8', $serialized) if $serialized;
+      } else {    # fragment case
+        $serialized = $result->toString(1, 1);
     } }
     elsif ($$opts{format} =~ /^html/) {
-      if (ref($result) =~ /^LaTeXML::(Post::)?Document$/) {    # Special for documents
-        $serialized = $result->getDocument->toStringHTML; }
-      else {                                                   # Regular for fragments
-        do {
-          local $XML::LibXML::setTagCompression = 1;
-          $serialized = $result->toString(1);
-        } } }
-    elsif ($$opts{format} eq 'dom') {
-      $serialized = $result; } }
-  else { $serialized = $result; }                              # Compressed case
+      if (ref($result) =~ /^LaTeXML::(Post::)?Document$/) {
+        # Needs explicit encode call, toStringHTML returns Perl byte strings
+        $serialized = $result->getDocument->toStringHTML;
+        $serialized = Encode::encode('UTF-8', $serialized) if $serialized; }
+      else {      # fragment case
+        local $XML::LibXML::setTagCompression = 1;
+        $serialized = $result->toString(1, 1); } } }
+  # Compressed/archive/other case, just pass on
+  else { $serialized = $result; }
 
   # 5.2 Finalize logging and return a response containing the document result, log and status
-  print STDERR "Status:conversion:" . ($$runtime{status_code} || '0') . " \n";
-  my $log = $self->flush_log;
-  return { result => $serialized, log => $log, status => $$runtime{status}, 'status_code' => $$runtime{status_code} };
+  Note("Status:conversion:" . ($$runtime{status_code} || '0'));
+  return { result => $serialized, log => $self->flush_log, status => $$runtime{status}, 'status_code' => $$runtime{status_code} };
 }
 
 ###########################################
 ####       Converter Management       #####
 ###########################################
+
+our %DAEMON_CACHE = ();
+our %CONFIG_CACHE = ();
+
 sub get_converter {
-  my ($self, $config) = @_;
-  # TODO: Make this more flexible via an admin interface later
-  my $key = $config->get('cache_key') || $config->get('profile') || 'custom';
-  my $d = $DAEMON_DB{$key};
-  if (!defined $d) {
-    $d = LaTeXML->new($config->clone);
-    $DAEMON_DB{$key} = $d; }
-  return $d; }
+  my ($self, $config, $key) = @_;
+  # Default key, unless made explicit
+  if (!$key && $config) {
+    $key = $config->get('cache_key') || $config->get('profile'); }
+
+  my $converter = $key && $DAEMON_CACHE{$key};
+
+  if (!defined $converter) {
+    # Trading flexibility for performance:
+    # once a cache_key is used, it can not be redefined in the same session
+    # (and we never needed it redefine-able)
+    #
+    # Instead, once a cache_key is specified, (re)booting converter objects
+    # for that config set can be done without spending time examining options
+    # via the $CONFIG_CACHE
+    $config ||= $CONFIG_CACHE{$key};
+
+    $converter = LaTeXML->new($config->clone);
+    if ($key) {
+      # cache both converter and config, for self-contained reloading
+      $DAEMON_CACHE{$key} = $converter;
+      $CONFIG_CACHE{$key} = $config; } }
+  return $converter; }
 
 ###########################################
 ####       Helper routines            #####
@@ -368,9 +422,9 @@ sub convert_post {
   my $runtime = $$self{runtime};
   my ($xslt, $parallel, $math_formats, $format, $verbosity, $defaultresources, $embed) =
     map { $$opts{$_} } qw(stylesheet parallelmath math_formats format verbosity defaultresources embed);
-  $verbosity = $verbosity || 0;
-
-  my %PostOPS = (verbosity => $verbosity,
+##  $verbosity = $verbosity || 0;
+  SetVerbosity($verbosity) if defined $verbosity;
+  my %PostOPS = (    ####verbosity => $verbosity,
     validate           => $$opts{validate},
     sourceDirectory    => $$opts{sourcedirectory},
     siteDirectory      => $$opts{sitedirectory},
@@ -389,7 +443,7 @@ sub convert_post {
   $parallel = $parallel || 0;
 
   my $DOCUMENT = LaTeXML::Post::Document->new($dom, %PostOPS);
-  my @procs = ();
+  my @procs    = ();
   #TODO: Add support for the following:
   my $dbfile = $$opts{dbfile};
   if (defined $dbfile && !-f $dbfile) {
@@ -401,7 +455,10 @@ sub convert_post {
     require LaTeXML::Post::Split;
     push(@procs, LaTeXML::Post::Split->new(split_xpath => $$opts{splitpath}, splitnaming => $$opts{splitnaming},
         db => $DB, %PostOPS)); }
-  my $scanner = ($$opts{scan} || $DB) && (LaTeXML::Post::Scan->new(db => $DB, %PostOPS));
+  my $scanner = ($$opts{scan} || $DB) && (LaTeXML::Post::Scan->new(
+      db       => $DB,
+      labelids => $$opts{splitnaming} && ($$opts{splitnaming} =~ /^label/),
+      %PostOPS));
   push(@procs, $scanner) if $$opts{scan};
   if (!($$opts{prescan})) {
     if ($$opts{index}) {
@@ -416,10 +473,10 @@ sub convert_post {
     if ($$opts{crossref}) {
       require LaTeXML::Post::CrossRef;
       push(@procs, LaTeXML::Post::CrossRef->new(
-          db => $DB, urlstyle => $$opts{urlstyle},
+          db        => $DB, urlstyle => $$opts{urlstyle},
           extension => $$opts{extension},
-          ($$opts{numbersections} ? (number_sections => 1) : ()),
-          ($$opts{navtoc} ? (navigation_toc => $$opts{navtoc}) : ()),
+          ($$opts{numbersections} ? (number_sections => 1)              : ()),
+          ($$opts{navtoc}         ? (navigation_toc  => $$opts{navtoc}) : ()),
           %PostOPS)); }
     if ($$opts{picimages}) {
       require LaTeXML::Post::PictureImages;
@@ -449,20 +506,20 @@ sub convert_post {
           require LaTeXML::Post::MathML::Presentation;
           push(@mprocs, LaTeXML::Post::MathML::Presentation->new(
               linelength => $$opts{linelength},
-              (defined $$opts{plane1} ? (plane1 => $$opts{plane1}) : (plane1 => 1)),
-              ($$opts{hackplane1} ? (hackplane1 => 1) : ()),
+              (defined $$opts{plane1} ? (plane1     => $$opts{plane1}) : (plane1 => 1)),
+              ($$opts{hackplane1}     ? (hackplane1 => 1)              : ()),
               %PostOPS)); }
         elsif ($fmt eq 'cmml') {
           require LaTeXML::Post::MathML::Content;
           push(@mprocs, LaTeXML::Post::MathML::Content->new(
-              (defined $$opts{plane1} ? (plane1 => $$opts{plane1}) : (plane1 => 1)),
-              ($$opts{hackplane1} ? (hackplane1 => 1) : ()),
+              (defined $$opts{plane1} ? (plane1     => $$opts{plane1}) : (plane1 => 1)),
+              ($$opts{hackplane1}     ? (hackplane1 => 1)              : ()),
               %PostOPS)); }
         elsif ($fmt eq 'om') {
           require LaTeXML::Post::OpenMath;
           push(@mprocs, LaTeXML::Post::OpenMath->new(
-              (defined $$opts{plane1} ? (plane1 => $$opts{plane1}) : (plane1 => 1)),
-              ($$opts{hackplane1} ? (hackplane1 => 1) : ()),
+              (defined $$opts{plane1} ? (plane1     => $$opts{plane1}) : (plane1 => 1)),
+              ($$opts{hackplane1}     ? (hackplane1 => 1)              : ()),
               %PostOPS)); }
         elsif ($fmt eq 'images') {
           require LaTeXML::Post::MathImages;
@@ -476,7 +533,7 @@ sub convert_post {
         elsif ($fmt eq 'mathtex') {
           require LaTeXML::Post::TeXMath;
           push(@mprocs, LaTeXML::Post::TeXMath->new(%PostOPS)); }
-        elsif ($fmt eq 'mathlex') {
+        elsif ($fmt eq 'lexemes') {
           require LaTeXML::Post::LexMath;
           push(@mprocs, LaTeXML::Post::LexMath->new(%PostOPS)); }
       }
@@ -491,7 +548,7 @@ sub convert_post {
     }
     if ($xslt) {
       require LaTeXML::Post::XSLT;
-      my $parameters = { LATEXML_VERSION => "'$LaTeXML::VERSION'" };
+      my $parameters  = { LATEXML_VERSION => "'$LaTeXML::VERSION'" };
       my @searchpaths = ('.', $DOCUMENT->getSearchPaths);
       # store these for the XSLT; XSLT Processor will copy resources where needed.
       foreach my $css (@{ $$opts{css} }) {
@@ -532,46 +589,44 @@ sub convert_post {
 
   # Do the actual post-processing:
   my @postdocs;
-  my $latexmlpost = LaTeXML::Post->new(verbosity => $verbosity || 0);
-  my $post_eval_return = eval {
-    local $SIG{'ALRM'} = sub { die "Fatal:conversion:post-processing timed out.\n" };
-    alarm($$opts{timeout});
+##  my $latexmlpost      = LaTeXML::Post->new(verbosity => $verbosity || 0);
+  my $latexmlpost = LaTeXML::Post->new();
+  eval {
+    alarm($$runtime{TTL});
     @postdocs = $latexmlpost->ProcessChain($DOCUMENT, @procs);
-    alarm(0);
-    1;
-  };
+    $$runtime{TTL} = alarm(0);
+    1; };
   # 3.1 Bookkeeping if a post-processing Fatal error occurred
-  local $@ = 'Fatal:conversion:unknown Post-processing failed! (Unknown Reason)'
-    if ((!$post_eval_return) && (!$@));
   if ($@) {    #Fatal occured!
     $$runtime{status_code} = 3;
-    $@ = 'Fatal:conversion:unknown ' . $@ unless $@ =~ /^\n?\S*Fatal:/s;
-    print STDERR $@;
-    undef @postdocs;    # Empty document for fatals, for sanity's sake
-  }
+    local $@ = 'Fatal:conversion:unknown ' . $@ unless $@ =~ /^\n?\S*Fatal:/s;
+    Debug($@); }
 
   # Finalize by arranging any manifests and packaging the output.
   # If our format requires a manifest, create one
   if (($$opts{whatsout} =~ /^archive/) && ($format !~ /^x?html|xml/)) {
     require LaTeXML::Post::Manifest;
-    my $manifest_maker = LaTeXML::Post::Manifest->new(db => $DB, format => $format, log=>$$opts{log}, %PostOPS);
+    my $manifest_maker = LaTeXML::Post::Manifest->new(db => $DB, format => $format, log => $$opts{log}, %PostOPS);
     $manifest_maker->process(@postdocs); }
   # Archives: when a relative --log is requested, write to sandbox prior packing
+  #     TODO: This can be enhanced, as any path can be relativized.
+  #     TODO: ALSO, we now always have a log file, so maybe always add it as default?
   if ($$opts{log} && ($$opts{whatsout} =~ /^archive/) && (!pathname_is_absolute($$opts{log}))) {
     ### We can't rely on the ->getDestinationDirectory method, as Fatal post-processing jobs have UNDEF @postdocs !!!
     ### my $destination_directory = $postdocs[0]->getDestinationDirectory();
     my $destination_directory = $PostOPS{destinationDirectory};
-    my $log_file = pathname_absolute($$opts{log}, $destination_directory);
+    my $log_file              = pathname_absolute($$opts{log}, $destination_directory);
     if (pathname_is_contained($log_file, $destination_directory)) {
-      print STDERR "\nPost-processing complete: " . $latexmlpost->getStatusMessage . "\n";
-      print STDERR "processing finished " . localtime() . "\n" if $verbosity >= 0;
+      NoteLog(($$opts{recursive} ? "recursive " : "") . "Post-processing complete: " . $latexmlpost->getStatusMessage);
+      NoteLog(($$opts{recursive} ? "recursive " : "") . "processing finished " . localtime());
       my $archive_log_status_code = max($$runtime{status_code}, $latexmlpost->getStatusCode);
-      print STDERR "Status:conversion:" . $archive_log_status_code . " \n";
+      Note("Status:conversion:" . $archive_log_status_code);
       open my $log_fh, '>', $log_file;
       print $log_fh $self->flush_log;
       close $log_fh;
       $self->bind_log; }
-    else { print STDERR "Error:I/O:log The target log file isn't contained in the destination directory!\n"; } }
+# TODO: This needs a bit of rethinking, likely a fallback.log file should be created and returned with the archive
+    else { Error("I/O", "log", "The target log file isn't contained in the destination directory!"); } }
   # Handle the output packaging
 
   my ($postdoc) = pack_collection(collection => [@postdocs], whatsout => $$opts{whatsout}, format => $format, %PostOPS);
@@ -586,9 +641,9 @@ sub convert_post {
   }
   $$runtime{status_code} = max($$runtime{status_code}, $latexmlpost->getStatusCode);
   ### HACKY END
+  Note(($$opts{recursive} ? "recursive " : "") . "Post-processing complete: " . $latexmlpost->getStatusMessage);
+  NoteLog(($$opts{recursive} ? "recursive " : "") . "processing finished " . localtime());
 
-  print STDERR "\nPost-processing complete: " . $latexmlpost->getStatusMessage . "\n";
-  print STDERR "processing finished " . localtime() . "\n" if $verbosity >= 0;
   # Avoid writing the main file twice (non-archive documents):
   if ($$opts{destination} && $$opts{local} && ($$opts{whatsout} eq 'document')) {
     undef $postdoc; }
@@ -608,15 +663,19 @@ sub new_latexml {
       push @pre, $pre;
     }
   }
+  my $includepathpis = !(exists $$opts{xsltparameters} &&
+    (grep { $_ eq 'LATEXML_VERSION:TEST' } @{ $$opts{xsltparameters} }));
   require LaTeXML;
+  SetVerbosity($$opts{verbosity}) if defined $$opts{verbosity};
   my $latexml = LaTeXML::Core->new(preload => [@pre], searchpaths => [@{ $$opts{paths} }],
-    graphicspaths   => ['.'],
-    verbosity       => $$opts{verbosity}, strict => $$opts{strict},
+    graphicspaths => ['.'],
+###    verbosity       => $$opts{verbosity}, strict => $$opts{strict},
     includecomments => $$opts{comments},
+    includepathpis  => $includepathpis,
     inputencoding   => $$opts{inputencoding},
     includestyles   => $$opts{includestyles},
     documentid      => $$opts{documentid},
-    nomathparse     => $$opts{nomathparse},                           # Backwards compatibility
+    nomathparse     => $$opts{nomathparse},     # Backwards compatibility
     mathparse       => $$opts{mathparse});
 
   if (my @baddirs = grep { !-d $_ } @{ $$opts{paths} }) {
@@ -629,42 +688,35 @@ sub new_latexml {
 
   # TODO: Do again, need to do this in a GOOD way as well:
   $latexml->digestFile($_, noinitialize => 1) foreach (@str_pre);
-  print STDERR "\n\n";    # Flush a pair of newlines to delimit the initalization
   return $latexml;
 }
 
 sub bind_log {
   my ($self) = @_;
-  # HACK HACK HACK !!! Refactor with proplery scoped logging !!!
-  $LaTeXML::LOG_STACK++;    # May the modern Perl community forgive me for this hack...
-  return if $LaTeXML::LOG_STACK > 1;
-  # TODO: Move away from global file handles, they will inevitably end up causing problems..
-
-  if (!$LaTeXML::DEBUG) {    # Debug will use STDERR for logs
-                             # Tie STDERR to log:
-    my $log_handle;
-    open($log_handle, ">>", \$$self{log}) or croak "Can't redirect STDERR to log! Dying...";
-    *STDERR_SAVED = *STDERR;
-    *STDERR       = *$log_handle;
-    binmode(STDERR, ':encoding(UTF-8)');
-    $LaTeXML::Common::Error::COLORIZED_LOGGING = -t STDERR;
-    $$self{log_handle} = $log_handle;
-  }
+  my $opts = $$self{opts};
+  # For now --whatsout=archive just gets deferred back to the old code
+  # But this needs a streamlined refactor in the future.
+  # Something to consider:
+  # 1. Multiple latexml_workers are started in the same CWD by their harness
+  # 2. Before an archive job starts, they would be opening the same default
+  # "latexml.log" or "cortex.log" etc.
+  # 3. The final log destination is available only after the temp dir is setup
+  # ... So we may want to completely remodel the whatsout handling
+  #     but what is cleanest and least confusing?
+  if (my $logfile = $$opts{log}) {
+    my $whatsout = $$opts{whatsout};
+    if ($whatsout && ($whatsout !~ /^archive/)) {
+      UseLog($logfile, 1); }
+    else {
+      UseLog(\$$self{log}, 1);
+    }
+  } else {
+    UseLog(\$$self{log}, 1); }
   return; }
 
 sub flush_log {
   my ($self) = @_;
-  # HACK HACK HACK !!! Refactor with proplery scoped logging !!!
-  $LaTeXML::LOG_STACK--;    # May the modern Perl community forgive me for this hack...
-  return '' if $LaTeXML::LOG_STACK > 0;
-
-  # Close and restore STDERR to original condition.
-  if (!$LaTeXML::DEBUG) {
-    close $$self{log_handle};
-    delete $$self{log_handle};
-    *STDERR                                    = *STDERR_SAVED;
-    $LaTeXML::Common::Error::COLORIZED_LOGGING = -t STDERR;
-  }
+  UseLog(undef);
   my $log = $$self{log};
   $$self{log} = q{};
   return $log; }
@@ -758,7 +810,7 @@ Binds STDERR to a "log" field in the $converter object
 =item C<< my $log = $converter->flush_log; >>
 
 Flushes out the accumulated conversion log into $log,
-         reseting STDERR to its usual stream.
+         resetting STDERR to its usual stream.
 
 =back
 
