@@ -25,22 +25,24 @@ use base qw(LaTeXML::Common::Object);
 # including basic font information, color & background color
 # as well as encoding and language information.
 
+DebuggableFeature('size-detailed', "Show sizing of boxes in detail");
 # NOTE: This is now in Common that it may evolve to be useful in Post processing...
 
 my $DEFFAMILY     = 'serif';      # [CONSTANT]
 my $DEFSERIES     = 'medium';     # [CONSTANT]
 my $DEFSHAPE      = 'upright';    # [CONSTANT]
 my $DEFCOLOR      = 'black';      # [CONSTANT]
-my $DEFBACKGROUND = 'white';      # [CONSTANT]
+my $DEFBACKGROUND = undef;        # [CONSTANT] no color; transparent
 my $DEFOPACITY    = '1';          # [CONSTANT]
 my $DEFENCODING   = 'OT1';        # [CONSTANT]
 my $DEFLANGUAGE   = undef;
 
 sub DEFSIZE { return $STATE->lookupValue('NOMINAL_FONT_SIZE') || 10; }
 
-my $FORCE_FAMILY = 0x1;
-my $FORCE_SERIES = 0x2;
-my $FORCE_SHAPE  = 0x4;
+my $FLAG_FORCE_FAMILY = 0x1;
+my $FLAG_FORCE_SERIES = 0x2;
+my $FLAG_FORCE_SHAPE  = 0x4;
+my $FLAG_EMPH         = 0x10;
 #======================================================================
 # Mappings from various forms of names or component names in TeX
 # Given a font, we'd like to map it to the "logical" names derived from LaTeX,
@@ -53,9 +55,9 @@ my $FORCE_SHAPE  = 0x4;
 # NOTE: This probably doesn't really belong in here...
 
 my %font_family = (
-  cmr  => { family => 'serif' },      cmss  => { family => 'sansserif' },
-  cmtt => { family => 'typewriter' }, cmvtt => { family => 'typewriter' },
-  cmti => { family => 'typewriter', shape => 'italic' },
+  cmr   => { family => 'serif' },      cmss  => { family => 'sansserif' },
+  cmtt  => { family => 'typewriter' }, cmvtt => { family => 'typewriter' },
+  cmti  => { family => 'typewriter', shape => 'italic' },
   cmfib => { family => 'serif' },      cmfr  => { family => 'serif' },
   cmdh  => { family => 'serif' },      cm    => { family => 'serif' },
   ptm   => { family => 'serif' },      ppl   => { family => 'serif' },
@@ -73,21 +75,21 @@ my %font_family = (
   pxr   => { family => 'serif' },      pxms  => { family => 'symbol' },
   pxsya => { family => 'symbol' },     pxsyb => { family => 'symbol' },
   futs  => { family => 'serif' },
-  uaq   => { family => 'serif' },      ugq   => { family => 'sansserif' },
-  eur   => { family => 'serif' },      eus   => { family => 'script' },
-  euf   => { family => 'fraktur' },    euex  => { family => 'symbol' },
+  uaq   => { family => 'serif' },   ugq  => { family => 'sansserif' },
+  eur   => { family => 'serif' },   eus  => { family => 'script' },
+  euf   => { family => 'fraktur' }, euex => { family => 'symbol' },
   # The following are actually math fonts.
   ms    => { family => 'symbol' },
-  ccm   => { family => 'serif', shape => 'italic' },
-  cmm   => { family => 'italic', encoding => 'OML' },
-  cmex  => { family => 'symbol', encoding => 'OMX' },       # Not really symbol, but...
-  cmsy  => { family => 'symbol', encoding => 'OMS' },
-  ccitt => { family => 'typewriter', shape => 'italic' },
-  cmbrm => { family => 'sansserif', shape => 'italic' },
-  futm  => { family => 'serif', shape => 'italic' },
-  futmi => { family => 'serif', shape => 'italic' },
-  txmi  => { family => 'serif', shape => 'italic' },
-  pxmi  => { family => 'serif', shape => 'italic' },
+  ccm   => { family => 'serif',      shape    => 'italic' },
+  cmm   => { family => 'italic',     encoding => 'OML' },
+  cmex  => { family => 'symbol',     encoding => 'OMX' },      # Not really symbol, but...
+  cmsy  => { family => 'symbol',     encoding => 'OMS' },
+  ccitt => { family => 'typewriter', shape    => 'italic' },
+  cmbrm => { family => 'sansserif',  shape    => 'italic' },
+  futm  => { family => 'serif',      shape    => 'italic' },
+  futmi => { family => 'serif',      shape    => 'italic' },
+  txmi  => { family => 'serif',      shape    => 'italic' },
+  pxmi  => { family => 'serif',      shape    => 'italic' },
   bbm   => { family => 'blackboard' },
   bbold => { family => 'blackboard' },
   bbmss => { family => 'blackboard' },
@@ -128,10 +130,10 @@ sub lookupFontShape {
 # extended logical font sizes, based on nominal document size of 10pts
 # Possibly should simply use absolute font point sizes, as declared in class...
 my %font_size = (
-  tiny   => 0.5, SMALL => 0.7, Small => 0.8,  small => 0.9,
-  normal => 1.0, large => 1.2, Large => 1.44, LARGE => 1.728,
-  huge => 2.074, Huge => 2.488,
-  big => 1.2, Big => 1.6, bigg => 2.1, Bigg => 2.6,
+  tiny   => 0.5,   SMALL => 0.7, Small => 0.8,  small => 0.9,
+  normal => 1.0,   large => 1.2, Large => 1.44, LARGE => 1.728,
+  huge   => 2.074, Huge  => 2.488,
+  big    => 1.2,   Big   => 1.6, bigg => 2.1, Bigg => 2.6,
 );
 
 sub rationalizeFontSize {
@@ -166,6 +168,7 @@ sub decodeFontname {
     $props{size} = $size;
     # Experimental Hack !?!?!?
     $props{encoding} = 'OT1' unless defined $props{encoding};
+    $props{at}       = $at . "pt" if defined $at;
     return %props; }
   else {
     return; } }
@@ -198,17 +201,18 @@ sub new {
   my $encoding  = $options{encoding};
   my $language  = $options{language};
   my $mathstyle = $options{mathstyle};
+
   if ($options{forcebold}) {    # for compatibility
     $series = 'bold'; $options{forceseries} = 1; }
-  my $force = 0
-    | ($options{forcefamily} ? $FORCE_FAMILY : 0)
-    | ($options{forceseries} ? $FORCE_SERIES : 0)
-    | ($options{forceshape}  ? $FORCE_SHAPE  : 0);
+  my $flags = 0
+    | ($options{forcefamily} ? $FLAG_FORCE_FAMILY : 0)
+    | ($options{forceseries} ? $FLAG_FORCE_SERIES : 0)
+    | ($options{forceshape}  ? $FLAG_FORCE_SHAPE  : 0);
   return $class->new_internal(
-    $family, $series, $shape, rationalizeFontSize($size),
-    $color, $bg, $opacity,
+    $family,    $series, $shape, rationalizeFontSize($size),
+    $color,     $bg,     $opacity,
     $encoding,  $language,
-    $mathstyle, $force); }
+    $mathstyle, $flags); }
 
 sub new_internal {
   my ($class, @components) = @_;
@@ -225,6 +229,7 @@ sub mathDefault {
     $DEFCOLOR, $DEFBACKGROUND, $DEFOPACITY, undef, $DEFLANGUAGE, 'text', 0); }
 
 # Accessors
+# Using an array here is getting ridiculous!
 sub getFamily     { my ($self) = @_; return $$self[0]; }
 sub getSeries     { my ($self) = @_; return $$self[1]; }
 sub getShape      { my ($self) = @_; return $$self[2]; }
@@ -235,6 +240,7 @@ sub getOpacity    { my ($self) = @_; return $$self[6]; }
 sub getEncoding   { my ($self) = @_; return $$self[7]; }
 sub getLanguage   { my ($self) = @_; return $$self[8]; }
 sub getMathstyle  { my ($self) = @_; return $$self[9]; }
+sub getFlags      { my ($self) = @_; return $$self[10]; }
 
 sub toString {
   my ($self) = @_;
@@ -243,17 +249,19 @@ sub toString {
 # Perhaps it is more useful to list only the non-default components?
 sub stringify {
   my ($self) = @_;
-  my ($fam, $ser, $shp, $siz, $col, $bkg, $opa, $enc, $lang, $mstyle) = @$self;
+  my ($fam, $ser, $shp, $siz, $col, $bkg, $opa, $enc, $lang, $mstyle, $flags) = @$self;
   $fam = 'serif' if $fam && ($fam eq 'math');
   return 'Font[' . join(',', map { Stringify($_) } grep { $_ }
-      (isDiff($fam, $DEFFAMILY) ? ($fam) : ()),
-    (isDiff($ser, $DEFSERIES)     ? ($ser) : ()),
-    (isDiff($shp, $DEFSHAPE)      ? ($shp) : ()),
-    (isDiff($siz, DEFSIZE())      ? ($siz) : ()),
-    (isDiff($col, $DEFCOLOR)      ? ($col) : ()),
-    (isDiff($bkg, $DEFBACKGROUND) ? ($bkg) : ()),
-    (isDiff($opa, $DEFOPACITY)    ? ($opa) : ()),
-    ($mstyle ? ($mstyle) : ()))
+      (isDiff($fam, $DEFFAMILY)   ? ($fam)    : ()),
+    (isDiff($ser, $DEFSERIES)     ? ($ser)    : ()),
+    (isDiff($shp, $DEFSHAPE)      ? ($shp)    : ()),
+    (isDiff($siz, DEFSIZE())      ? ($siz)    : ()),
+    (isDiff($col, $DEFCOLOR)      ? ($col)    : ()),
+    (isDiff($bkg, $DEFBACKGROUND) ? ($bkg)    : ()),
+    (isDiff($opa, $DEFOPACITY)    ? ($opa)    : ()),
+    ($mstyle                      ? ($mstyle) : ()),
+    ($flags                       ? ($flags)  : ()),
+    )
     . ']'; }
 
 sub equals {
@@ -277,12 +285,13 @@ sub match {
 
 sub makeConcrete {
   my ($self, $concrete) = @_;
-  my ($family, $series, $shape, $size, $color, $bg, $opacity, $encoding, $lang, $mstyle) = @$self;
-  my ($ofamily, $oseries, $oshape, $osize, $ocolor, $obg, $oopacity, $oencoding, $olang, $omstyle) = @$concrete;
+  my ($family, $series, $shape, $size, $color, $bg, $opacity, $encoding, $lang, $mstyle, $flags) = @$self;
+  my ($ofamily, $oseries, $oshape, $osize, $ocolor, $obg, $oopacity, $oencoding, $olang, $omstyle, $oflags) = @$concrete;
   return (ref $self)->new_internal(
-    $family || $ofamily, $series || $oseries, $shape || $oshape, $size || $osize,
-    $color || $ocolor, $bg || $obg, (defined $opacity ? $opacity : $oopacity),
-    $encoding || $oencoding, $lang || $olang, $mstyle || $omstyle); }
+    $family   || $ofamily,   $series || $oseries, $shape || $oshape, $size || $osize,
+    $color    || $ocolor,    $bg     || $obg, (defined $opacity ? $opacity : $oopacity),
+    $encoding || $oencoding, $lang   || $olang, $mstyle || $omstyle,
+    ($flags || 0) | ($oflags || 0)); }
 
 sub isDiff {
   my ($x, $y) = @_;
@@ -296,12 +305,19 @@ sub isDiff {
 # Namely, the result is a hash keyed on the attribute name and whose value is a hash
 #    value      => "new_attribute_value"
 #    properties => { %fontproperties }
+# Note in particular 2 interesting keys
+#   element: can specify the element tagname to use for wrapping instead of ltx:text
+#   class: can be used to add a class attribute to the wrapping element
 sub relativeTo {
   my ($self, $other) = @_;
-  my ($fam,  $ser,  $shp,  $siz,  $col,  $bkg,  $opa,  $enc,  $lang,  $mstyle)  = @$self;
-  my ($ofam, $oser, $oshp, $osiz, $ocol, $obkg, $oopa, $oenc, $olang, $omstyle) = @$other;
+  my ($fam,  $ser,  $shp,  $siz,  $col,  $bkg,  $opa,  $enc,  $lang,  $mstyle,  $flags)  = @$self;
+  my ($ofam, $oser, $oshp, $osiz, $ocol, $obkg, $oopa, $oenc, $olang, $omstyle, $oflags) = @$other;
   $fam  = 'serif' if $fam  && ($fam eq 'math');
   $ofam = 'serif' if $ofam && ($ofam eq 'math');
+##  my $emph = 0;
+##  $emph ||= $shp && $shp =~ s/^emph-//;
+## ##  $emph ||= $oshp && $oshp =~ s/^emph-//;
+##   $oshp && $oshp =~ s/^emph-//;
   my @diffs = (
     (isDiff($fam, $ofam) ? ($fam) : ()),
     (isDiff($ser, $oser) ? ($ser) : ()),
@@ -329,18 +345,24 @@ sub relativeTo {
     (isDiff($lang, $olang)
       ? ('xml:lang' => { value => $lang, properties => { language => $lang } })
       : ()),
+    (!$mstyle && $flags && ($flags & $FLAG_EMPH) && (!$oflags || !($oflags & $FLAG_EMPH))
+      ? (
+        #         class => { value => 'ltx_emph' },
+        element => { value => 'ltx:emph' }
+        )
+      : ()),
     ### Contemplate this: We do NOT want mathstyle showing up (automatically) in the attributes
     ### So, we presumably want to ignore differences in mathstyle
     ### They shouldn't (by themselves) affect the display?
 ###    (isDiff($mstyle, $omstyle)
 ###      ? ('mathstyle' => { value => $mstyle, properties => { mathstyle => $mstyle } })
 ###      : ()),
-    ); }
+  ); }
 
 sub distance {
   my ($self, $other) = @_;
-  my ($fam,  $ser,  $shp,  $siz,  $col,  $bkg,  $opa,  $enc,  $lang,  $mstyle)  = @$self;
-  my ($ofam, $oser, $oshp, $osiz, $ocol, $obkg, $oopa, $oenc, $olang, $omstyle) = @$other;
+  my ($fam,  $ser,  $shp,  $siz,  $col,  $bkg,  $opa,  $enc,  $lang,  $mstyle,  $flags)  = @$self;
+  my ($ofam, $oser, $oshp, $osiz, $ocol, $obkg, $oopa, $oenc, $olang, $omstyle, $oflags) = @$other;
   $fam  = 'serif' if $fam  && ($fam eq 'math');
   $ofam = 'serif' if $ofam && ($ofam eq 'math');
   return
@@ -355,6 +377,7 @@ sub distance {
     + (isDiff($lang, $olang) ? 1 : 0)
     # Let's not consider mathstyle differences here, either.
 ###    + (isDiff($mstyle, $omstyle) ? 1 : 0)
+    + (($flags & $FLAG_EMPH) ^ ($oflags & $FLAG_EMPH) ? 1 : 0)
     ; }
 
 # This matches fonts when both are converted to strings (toString),
@@ -371,10 +394,9 @@ sub match_font {
   if (!$regexp) {
     if ($font1 =~ /^Font\[(.*)\]$/) {
       my @comp = split(',', $1);
-      my $re = '^Font\['
+      my $re   = '^Font\['
         . join(',', map { ($_ eq '*' ? "[^,]+" : "\Q$_\E") } @comp)
         . '\]$';
-      print STDERR "\nCreating re for \"$font1\" => $re\n";
       $regexp = $FONT_REGEXP_CACHE{$font1} = qr/$re/; } }
   return $font2 =~ /$regexp/; }
 
@@ -425,16 +447,18 @@ our %mathstylesize = (display => 1, text => 1,
 # NOTE: that we assume the size has already been adjusted for mathstyle, if necessary.
 sub computeStringSize {
   my ($self, $string) = @_;
+  if ((!defined $string) || ($string eq '') || ($self->getFamily eq 'nullfont')) {
+    return (Dimension(0), Dimension(0), Dimension(0)); }
   my $size = ($self->getSize || DEFSIZE() || 10); ## * $mathstylesize{ $self->getMathstyle || 'text' };
-  my $l = (defined $string ? length($string) : 0);
-  my $u = $size * 65535;
+  my $l    = (defined $string ? length($string) : 0);
+  my $u    = $size * 65535;
   return (Dimension(0.75 * $u * $l), Dimension(0.7 * $u), Dimension(0.2 * $u)); }
 
 # Get nominal width, height base ?
 sub getNominalSize {
   my ($self) = @_;
   my $size = ($self->getSize || DEFSIZE() || 10); ## * $mathstylesize{ $self->getMathstyle || 'text' };
-  my $u = $size * 65535;
+  my $u    = $size * 65535;
   return (Dimension(0.75 * $u), Dimension(0.7 * $u), Dimension(0.2 * $u)); }
 
 # Here's where I avoid trying to emulate Knuth's line-breaking...
@@ -456,47 +480,44 @@ sub getNominalSize {
 # requested vs real sizes?
 sub computeBoxesSize {
   my ($self, $boxes, %options) = @_;
-  my $font = (ref $self ? $self : $STATE->lookupValue('font'));
+  my $font      = (ref $self ? $self : $STATE->lookupValue('font'));
   my $fillwidth = $options{width};
   if ((!defined $fillwidth) && ($fillwidth = $STATE->lookupDefinition(T_CS('\textwidth')))) {
     $fillwidth = $fillwidth->valueOf; }    # get register
   my $maxwidth = $fillwidth && $fillwidth->valueOf;
-  my @lines = ();
-  my ($wd, $ht, $dp) = (0, 0, 0);
+  my @lines    = ();
+  my ($wd, $ht, $dp)          = (0, 0, 0);
+  my ($minwd, $minht, $mindp) = (0, 0, 0);
   my $vattach = $options{vattach} || 'baseline';
-  foreach my $box (@$boxes) {
+  no warnings 'recursion';
+  # Flatten top-level Lists (orrr pass-thru $fillwidth ???)
+  my @boxes = map { (ref $_ eq 'LaTeXML::Core::List' ? $_->unlist : $_); } @$boxes;
+  foreach my $box (@boxes) {
     next unless defined $box;
     next if ref $box && !$box->can('getSize');    # Care!! Since we're asking ALL args/compoments
-    my ($w, $h, $d) = (ref $box ? $box->getSize(%options) : $font->computeStringSize($box));
+    ## Should any %options be inherited by the contained boxes?
+    my ($w, $h, $d) = (ref $box ? $box->getSize() : $font->computeStringSize($box));
     if (ref $w) {
       $wd += $w->valueOf; }
     else {
       Warn('expected', 'Dimension', undef,
-        "Width of " . Stringify($box) . " yeilded a non-dimension: " . Stringify($w)); }
+        "Width of " . Stringify($box) . " yielded a non-dimension: " . Stringify($w)); }
     if (ref $h) {
       $ht = max($ht, $h->valueOf); }
     else {
       Warn('expected', 'Dimension', undef,
-        "Height of " . Stringify($box) . " yeilded a non-dimension: " . Stringify($h)); }
+        "Height of " . Stringify($box) . " yielded a non-dimension: " . Stringify($h)); }
     if (ref $d) {
       $dp = max($dp, $d->valueOf); }
     else {
       Warn('expected', 'Dimension', undef,
-        "Depth of " . Stringify($box) . " yeilded a non-dimension: " . Stringify($d)); }
+        "Depth of " . Stringify($box) . " yielded a non-dimension: " . Stringify($d)); }
     if ((($options{layout} || '') eq 'vertical')    # EVERY box is a row?
-                                                    # || $box is a <ltx:break> (or similar)!!!!
-      ) {
+      || ((ref $box) && $box->getProperty('isBreak'))) {    # || $box is a linebreak
+      push(@lines, [$wd, $ht, $dp]); $wd = $ht = $dp = 0; }
+    elsif ((defined $maxwidth) && ($wd >= $maxwidth)) {     # or we've reached the requested width
+          # Instead of a real linebreaking algorithm, just break off if too wide.
       push(@lines, [$wd, $ht, $dp]); $wd = $ht = $dp = 0; }
-    elsif ((defined $maxwidth) && ($wd >= $maxwidth)) {    # or we've reached the requested width
-          # Compounding errors with wild abandon.
-          # If an underlying box is too wide, we'll split it up into multiple rows
-          # [Rather than correctly break it?]
-          # BUT How do we know if it should break at alL!?!?!?!?!
-##     while ($wd >= $maxwidth) {
-##       push(@lines, [$maxwidth, $ht, $dp]); $wd = $wd - $maxwidth; }
-##      $ht = $h->valueOf; $dp = $d->valueOf;     # continue with the leftover
-      push(@lines, [$wd, $ht, $dp]); $wd = $ht = $dp = 0;
-    }
   }
   if ($wd) {    # be sure to get last line
     push(@lines, [$wd, $ht, $dp]); }
@@ -510,7 +531,7 @@ sub computeBoxesSize {
     $dp = sum(map { $$_[2] } @lines);
     if ($vattach eq 'top') {    # Top of box is aligned with top(?) of current text
       my ($w, $h, $d) = $font->getNominalSize;
-      $h = $h->valueOf;
+      $h  = $h->valueOf;
       $dp = $ht + $dp - $h; $ht = $h; }
     elsif ($vattach eq 'bottom') {    # Bottom of box is aligned with bottom (?) of current text
       $ht = $ht + $dp; $dp = 0; }
@@ -522,10 +543,19 @@ sub computeBoxesSize {
     else {                            # default is baseline (of the 1st line)
       my $h = $lines[0][1];
       $dp = $ht + $dp - $h; $ht = $h; } }
+  $wd = max($minwd, $wd); $ht = max($minht, $ht); $dp = max($mindp, $dp);
   #print "BOXES SIZE ".($wd/65536)." x ".($ht/65536)." + ".($dp/65336)." for "
   #  .join(' ',grep {$_} map { Stringify($_) } @$boxes)."\n";
+  Debug("Size boxes " . join(',', map { $_ . '=' . ToString($options{$_}); } sort keys %options) . "\n"
+      . "  Boxes: " . join(',',  map { '[[' . ToString($_) . ']]'; } @$boxes) . "\n"
+      . "  Sizes: " . join("\n", map { _showsize(@$_); } @lines) . "\n"
+      . "  => " . _showsize($wd, $ht, $dp)) if $LaTeXML::DEBUG{'size-detailed'};
   return (Dimension($wd), Dimension($ht), Dimension($dp)); }
 
+sub _showsize {
+  my ($wd, $ht, $dp) = @_;
+  return ($wd / 65536) . " x " . ($ht / 65536) . " + " . ($dp / 65336); }
+
 sub isSticky {
   my ($self) = @_;
   return $$self[0] && ($$self[0] =~ /^(?:serif|sansserif|typewriter)$/); }
@@ -557,33 +587,34 @@ sub merge {
   my $encoding  = $options{encoding};
   my $language  = $options{language};
   my $mathstyle = $options{mathstyle};
+
   if ($options{forcebold}) {    # for compatibility
     $series = 'bold'; $options{forceseries} = 1; }
-  my $force = 0
-    | ($options{forcefamily} ? $FORCE_FAMILY : 0)
-    | ($options{forceseries} ? $FORCE_SERIES : 0)
-    | ($options{forceshape}  ? $FORCE_SHAPE  : 0);
+  my $flags = 0
+    | ($options{forcefamily} ? $FLAG_FORCE_FAMILY : 0)
+    | ($options{forceseries} ? $FLAG_FORCE_SERIES : 0)
+    | ($options{forceshape}  ? $FLAG_FORCE_SHAPE  : 0);
 
-  my $oforce = $$self[10];
+  my $oflags = $$self[10];
   # Fallback to positional invocation:
-  $family = $$self[0] if (!defined $family) || ($oforce & $FORCE_FAMILY);
-  $series = $$self[1] if (!defined $series) || ($oforce & $FORCE_SERIES);
-  $shape  = $$self[2] if (!defined $shape)  || ($oforce & $FORCE_SHAPE);
-  $size   = $$self[3] if (!defined $size);
-  $color  = $$self[4] if (!defined $color);
-  $bg     = $$self[5] if (!defined $bg);
+  $family    = $$self[0] if (!defined $family) || ($oflags & $FLAG_FORCE_FAMILY);
+  $series    = $$self[1] if (!defined $series) || ($oflags & $FLAG_FORCE_SERIES);
+  $shape     = $$self[2] if (!defined $shape)  || ($oflags & $FLAG_FORCE_SHAPE);
+  $size      = $$self[3] if (!defined $size);
+  $color     = $$self[4] if (!defined $color);
+  $bg        = $$self[5] if (!defined $bg);
   $opacity   = $$self[6] if (!defined $opacity);
   $encoding  = $$self[7] if (!defined $encoding);
   $language  = $$self[8] if (!defined $language);
   $mathstyle = $$self[9] if (!defined $mathstyle);
-  $force = ($$self[10] || 0) | $force;
+  $flags     = ($$self[10] || 0) | $flags;
 
   if (my $scale = $options{scale}) {
     $size = $scale * $size; }
   # Set the mathstyle, and also the size from the mathstyle
   # But we may need to scale that size against the existing or requested size.
   my $stylescale = ($$self[3] ? $$self[3] / $stylesize{ $$self[9] || 'display' } : 1);
-  if ($options{size}) { }    # Explicitly requested size, use it
+  if    ($options{size}) { }       # Explicitly requested size, use it
   elsif ($options{mathstyle}) {    # otherwise set the size from mathstyle
     $size = $stylescale * $stylesize{$mathstyle}; }
   elsif ($options{scripted}) {     # Or adjust both the mathstyle & size for scripts
@@ -594,12 +625,14 @@ sub merge {
     $size      = $stylescale * $stylesize{ $mathstyle || 'display' }; }
 
   if ($options{emph}) {
-    $shape = ($shape eq 'italic' ? 'upright' : 'italic'); }
+    $shape = ($shape eq 'italic' ? 'upright' : 'italic');
+    $flags |= $FLAG_EMPH; }
+  $flags &= ~$FLAG_EMPH if $mathstyle;    # Disable emph in math
 
   my $newfont = (ref $self)->new_internal($family, $series, $shape, $size,
-    $color, $bg, $opacity,
+    $color,     $bg, $opacity,
     $encoding,  $language,
-    $mathstyle, $force);
+    $mathstyle, $flags);
   if (my $specialize = $options{specialize}) {
     $newfont = $newfont->specialize($specialize); }
   return $newfont; }
@@ -616,33 +649,33 @@ sub specialize {
   my ($self, $string) = @_;
   return $self if !(defined $string) || ref $string;    # ?
   my ($family, $series, $shape, $size, $color, $bg, $opacity,
-    $encoding, $language, $mathstyle, $force) = @$self;
-  my $deffamily = ($force & $FORCE_FAMILY ? $family || $DEFFAMILY : $DEFFAMILY);
-  my $defseries = ($force & $FORCE_SERIES ? $series || $DEFSERIES : $DEFSERIES);
-  my $defshape  = ($force & $FORCE_SHAPE  ? $shape  || $DEFSHAPE  : $DEFSHAPE);
+    $encoding, $language, $mathstyle, $flags) = @$self;
+  my $deffamily = ($flags & $FLAG_FORCE_FAMILY ? $family || $DEFFAMILY : $DEFFAMILY);
+  my $defseries = ($flags & $FLAG_FORCE_SERIES ? $series || $DEFSERIES : $DEFSERIES);
+  my $defshape  = ($flags & $FLAG_FORCE_SHAPE  ? $shape  || $DEFSHAPE  : $DEFSHAPE);
   if (($string =~ /^\p{Latin}$/) && ($string =~ /^\p{L}$/)) {    # Latin Letter
     $shape = 'italic' if !$shape && !$family; }
   elsif ($string =~ /^\p{Greek}$/) {                             # Single Greek character?
     if ($string =~ /^\p{Lu}$/) {                                 # Uppercase
       if (!$family || ($family eq 'math')) {
         $family = $deffamily;
-        $shape = $defshape if $shape && ($shape ne $DEFSHAPE); } }    # if ANY shape, must be default
-    else {    # Lowercase
+        $shape  = $defshape if $shape && ($shape ne $DEFSHAPE); } }    # if ANY shape, must be default
+    else {                                                             # Lowercase
       $family = $deffamily if !$family || ($family ne $DEFFAMILY);
-      $shape  = 'italic'   if !$shape  || !($force & $FORCE_SHAPE);    # always ?
+      $shape  = 'italic'   if !$shape  || !($flags & $FLAG_FORCE_SHAPE);    # always ?
       if ($series && ($series ne $DEFSERIES)) { $series = $defseries; }
-    } }
-  elsif ($string =~ /^\p{N}$/) {                                       # Digit
+  } }
+  elsif ($string =~ /^\p{N}$/) {                                            # Digit
     if (!$family || ($family eq 'math')) {
       $family = $deffamily;
-      $shape  = $defshape; } }                                         # defaults, always.
-  else {                                                               # Other Symbol
+      $shape  = $defshape; } }                                              # defaults, always.
+  else {                                                                    # Other Symbol
     $family = $deffamily;
-    $shape  = $defshape;                                               # defaults, always.
+    $shape  = $defshape;                                                    # defaults, always.
     if ($series && ($series ne $DEFSERIES)) { $series = $defseries; } }
   return (ref $self)->new_internal($family, $series, $shape, $size,
-    $color, $bg, $opacity,
-    $encoding, $language, $mathstyle, $force); }
+    $color,    $bg, $opacity,
+    $encoding, $language, $mathstyle, $flags); }
 
 # A special form of merge when copying/moving nodes to a new context,
 # particularly math which become scripts or such.
@@ -665,23 +698,24 @@ sub purestyleChanges {
   my ($self, $other) = @_;
   my $mathstyle      = $self->getMathstyle;
   my $othermathstyle = $other->getMathstyle;
+  my $othercolor     = $other->getColor;
   return (
-    scale      => $other->getSize / $self->getSize,
-    color      => $other->getColor,
+    scale => $other->getSize / $self->getSize,
+    (isDiff($othercolor, $DEFCOLOR) ? (color => $othercolor) : ()),
     background => $other->getBackground,
-    opacity    => $other->getOpacity,                 # should multiply or replace?
+    opacity    => $other->getOpacity,      # should multiply or replace?
     ($mathstyle && $othermathstyle
       ? (mathstylestep => $mathstylestep{$mathstyle}{$othermathstyle})
       : ()),
-    ); }
+  ); }
 
 sub mergePurestyle {
   my ($self, %stylechanges) = @_;
   my $new = $self->new_internal(@$self);
-  $$new[3] = $$self[3] * $stylechanges{scale} if $stylechanges{scale};
-  $$new[4] = $stylechanges{color}             if $stylechanges{color};
-  $$new[5] = $stylechanges{background}        if $stylechanges{background};
-  $$new[6] = $stylechanges{opacity}           if $stylechanges{opacity};
+  $$new[3] = $$self[3] * $stylechanges{scale}                            if $stylechanges{scale};
+  $$new[4] = $stylechanges{color}                                        if $stylechanges{color};
+  $$new[5] = $stylechanges{background}                                   if $stylechanges{background};
+  $$new[6] = $stylechanges{opacity}                                      if $stylechanges{opacity};
   $$new[9] = $stepmathstyle{ $$self[9] }{ $stylechanges{mathstylestep} } if $stylechanges{mathstylestep};
   return new; }
 
@@ -690,7 +724,7 @@ sub mergePurestyle {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
