@@ -29,11 +29,15 @@ use LaTeXML::Core::MuGlue;
 use base qw(LaTeXML::Common::Object);
 #**********************************************************************
 sub new {
-  my ($class) = @_;
+  my ($class, %options) = @_;
   return bless {
-    mouth => undef, mouthstack => [], pushback => [], autoclose => 1, pending_comments => []
+    mouth     => undef, mouthstack => [], pushback => [], autoclose => 1, pending_comments => [],
+    verbosity => $options{verbosity} || 0,
+    progress  => 0,
   }, $class; }
 
+our $TOKEN_PROGRESS_QUANTUM = 30000;
+
 #**********************************************************************
 # Start reading tokens from a new Mouth.
 # This pushes the mouth as the current source that $gullet->readToken (etc) will read from.
@@ -59,10 +63,8 @@ sub closeMouth {
   if (@{ $$self{mouthstack} }) {
     ($$self{mouth}, $$self{pushback}, $$self{autoclose}) = @{ shift(@{ $$self{mouthstack} }) }; }
   else {
-    $$self{pushback} = [];
-##    $$self{mouth}=Tokens();
-    $$self{mouth} = LaTeXML::Core::Mouth->new();
-####    $$self{mouth}=undef;
+    $$self{pushback}  = [];
+    $$self{mouth}     = LaTeXML::Core::Mouth->new();
     $$self{autoclose} = 1; }
   return; }
 
@@ -76,12 +78,14 @@ sub mouthIsOpen {
     || grep { $_ && ($$_[0] eq $mouth) } @{ $$self{mouthstack} }; }
 
 # This flushes a mouth so that it will be automatically closed, next time it's read
-# Corresponds (I think) to TeX's \endinput
+# Corresponds to TeX's \endinput
 sub flushMouth {
   my ($self) = @_;
-  $$self{mouth}->finish;    # but not close!
-  $$self{pushback}  = [];   # And don't read anytyhing more from it.
-  $$self{autoclose} = 1;
+  my $mouth = $$self{mouth};
+  # Put the remainder of Mouth's current line at the END of the pushback stack, to be read
+  while (!$mouth->isEOL) {
+    push(@{ $$self{pushback} }, $mouth->readToken); }
+  $mouth->finish;    # then finish the mouth (it'll get closed on next read)
   return; }
 
 # Obscure, but the only way I can think of to End!! (see \bye or \end{document})
@@ -92,14 +96,21 @@ sub flush {
   while (@{ $$self{mouthstack} }) {
     my $entry = shift @{ $$self{mouthstack} };
     $$entry[0]->finish; }
-  $$self{pushback} = [];
-##  $$self{mouth}=Tokens();
-  $$self{mouth} = LaTeXML::Core::Mouth->new();
-####    $$self{mouth}=undef;
-  $$self{autoclose}  = 1;
+  $$self{pushback}   = [];
+  $$self{mouth}      = LaTeXML::Core::Mouth->new();
   $$self{mouthstack} = [];
   return; }
 
+sub setup_scan {
+  my ($self) = @_;
+  if ($$self{pushback_has_smuggled_the}) {
+    $$self{pushback_has_smuggled_the} = 0;
+    # setup new scan by removing any smuggle CCs
+    for my $token (@{ $$self{pushback} }) {
+      if ($$token[1] == CC_SMUGGLE_THE) {
+        $token = $$token[2]; } } }
+  return; }
+
 # Do something, while reading stuff from a specific Mouth.
 # This reads ONLY from that mouth (or any mouth openned by code in that source),
 # and the mouth should end up empty afterwards, and only be closed here.
@@ -154,7 +165,7 @@ sub getSource {
 
 sub getSourceMouth {
   my ($self) = @_;
-  my $mouth = $$self{mouth};
+  my $mouth  = $$self{mouth};
   my $source = defined $mouth && $mouth->getSource;
   if (!defined($source)) {
     foreach my $frame (@{ $$self{mouthstack} }) {
@@ -168,7 +179,10 @@ sub showUnexpected {
   my ($self) = @_;
   my $message = "Input is empty";
   if (my $token = $self->readToken) {
-    $message = "Next token is " . Stringify($token);
+    my @pb = @{ $$self{pushback} };
+    $message = "Next token is " . Stringify($token)
+      . " ( == " . Stringify($STATE->lookupMeaning($token)) . ")"
+      . (@pb ? " more: " . ToString(Tokens(@pb)) : '');
     unshift(@{ $$self{pushback} }, $token);
   }
   return $message; }
@@ -179,50 +193,107 @@ sub show_pushback {
   @pb = (@pb[0 .. 50], T_OTHER('...')) if scalar(@pb) > 55;
   return (@pb ? "\n  To be read again " . ToString(Tokens(@pb)) : ''); }
 
-#**********************************************************************
-# Not really 100% sure how this is supposed to work
-# See TeX Ch 20, p216 regarding noexpand, \edef with token list registers, etc.
-# Solution: Duplicate param tokens, stick NOTEXPANDED infront of expandable tokens.
-sub neutralizeTokens {
-  my ($self, @tokens) = @_;
-  my @result = ();
-  foreach my $token (@tokens) {
-    if ($$token[1] == CC_PARAM) {    # Inline ->getCatcode!
-      push(@result, $token); }
-    elsif (defined(my $defn = LaTeXML::Core::State::lookupDefinition($STATE, $token))) {
-      push(@result, Token('\noexpand', CC_NOTEXPANDED)); }
-    push(@result, $token); }
-  return @result; }
-
 #**********************************************************************
 # Low-level readers: read token, read expanded token
 #**********************************************************************
 # Note that every char (token) comes through here (maybe even twice, through args parsing),
 # So, be Fast & Clean!  This method only reads from the current input stream (Mouth).
-our @hold_token = (
+our @CATCODE_HOLD = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 1, 0,
-  0, 0, 1);
+  0, 1, 0, 0);
+
+sub handleMarker {
+  my ($self, $markertoken) = @_;
+  my $arg = $$markertoken[0];
+  if (ref $arg) {
+    LaTeXML::Core::Definition::stopProfiling($markertoken, 'expand'); }
+  elsif ($arg eq 'before-column') {    # Were in before-column template
+    my $alignment = $STATE->lookupValue('Alignment');
+    Debug("Halign $alignment: alignment state => 0") if $LaTeXML::DEBUG{halign};
+    $LaTeXML::ALIGN_STATE = 0; }       # switch to column proper!
+  elsif ($arg eq 'after-column') {     # Were in before-column template
+    my $alignment = $STATE->lookupValue('Alignment');
+    Debug("Halign $alignment: alignment state: after column") if $LaTeXML::DEBUG{halign};
+  }
+  return; }
+
+sub handleTemplate {
+  my ($self, $alignment, $token, $type, $hidden) = @_;
+  Debug("Halign $alignment: ALIGNMENT Column ended at " . Stringify($token)
+      . " type $type [" . Stringify($STATE->lookupMeaning($token)) . "]"
+      . "@ " . ToString($self->getLocator))
+    if $LaTeXML::DEBUG{halign};
+  #  Append expansion to end!?!?!?!
+  local $LaTeXML::CURRENT_TOKEN = $token;
+  my $post = $alignment->getColumnAfter;
+  $LaTeXML::ALIGN_STATE = 1000000;
+  ### NOTE: Truly fishy smuggling w/ \hidden@cr
+  my $arg;
+  if (($type eq 'cr') && $hidden) {    # \hidden@cr gets an argument as payload!!!!!
+    $arg = $self->readArg(); }
+  Debug("Halign $alignment: column after " . ToString($post)) if $LaTeXML::DEBUG{halign};
+  if ((($type eq 'cr') || ($type eq 'crcr'))
+    && $$alignment{in_row} && !$alignment->currentRow->{pseudorow}) {
+    unshift(@{ $$self{pushback} }, T_CS('\@row@after')); }
+  if ($arg) {
+    unshift(@{ $$self{pushback} }, T_BEGIN, $arg->unlist, T_END); }
+  unshift(@{ $$self{pushback} }, $token);
+  unshift(@{ $$self{pushback} }, $post->unlist);
+  return; }
+
+# If it is a column ending token, Returns the token, a keyword and whether it is "hidden"
+our @column_ends = (
+  [T_ALIGN, 'align', 0],
+  [T_CS('\cr'),           'cr',     0],
+  [T_CS('\crcr'),         'crcr',   0],
+  [T_CS('\hidden@cr'),    'cr',     1],
+  [T_CS('\hidden@crcr'),  'crcr',   1],
+  [T_CS('\hidden@align'), 'insert', 1],
+  [T_CS('\span'),         'span',   0]);
+
+sub isColumnEnd {
+  my ($self, $token) = @_;
+  my $cc = $$token[1];
+  return unless ($cc == CC_ALIGN) || ($cc == CC_CS);
+  # Embedded version of Equals, knowing both are tokens
+  my $defn = $STATE->lookupMeaning($token) || $token;
+  foreach my $end (@column_ends) {
+    my $e = $$end[0];
+    # Would be nice to cache the defns, but don't know when they're present & constant!
+    return @$end if $defn->equals($STATE->lookupMeaning($e) || $e); }
+  return; }
 
 sub readToken {
   my ($self) = @_;
   #  my $token = shift(@{$$self{pushback}});
-  my $token;
-  my $cc;
-  # Check in pushback first....
-  while (($token = shift(@{ $$self{pushback} })) && $hold_token[$cc = $$token[1]]) {
-    if ($cc == CC_COMMENT) {
-      push(@{ $$self{pending_comments} }, $token); }
-    elsif ($cc == CC_MARKER) {
-      LaTeXML::Core::Definition::stopProfiling($token, 'expand'); } }
-  return $token if defined $token;
-  while (($token = $$self{mouth}->readToken()) && $hold_token[$cc = $$token[1]]) {
-    if ($cc == CC_COMMENT) {
-      push(@{ $$self{pending_comments} }, $token); }    # What to do with comments???
-    elsif ($cc == CC_MARKER) {
-      LaTeXML::Core::Definition::stopProfiling($token, 'expand'); } }
+  my ($token, $cc, $atoken, $atype, $ahidden);
+  while (1) {
+    while (($token = shift(@{ $$self{pushback} }))
+      && (($$token[1] != CC_SMUGGLE_THE) || ($token = $$token[2]))
+      && $CATCODE_HOLD[$cc = $$token[1]]) {
+      if ($cc == CC_COMMENT) {
+        push(@{ $$self{pending_comments} }, $token); }
+      elsif ($cc == CC_MARKER) {
+        $self->handleMarker($token); } }
+    # Not in pushback, use the current mouth
+    if (!defined $token) {
+      while (($token = $$self{mouth}->readToken()) && $CATCODE_HOLD[$cc = $$token[1]]) {
+        if ($cc == CC_COMMENT) {
+          push(@{ $$self{pending_comments} }, $token); }    # What to do with comments???
+        elsif ($cc == CC_MARKER) {
+          $self->handleMarker($token); } } }
+    ProgressStep() if ($$self{progress}++ % $TOKEN_PROGRESS_QUANTUM) == 0;
+    # Wow!!!!! See TeX the Program \S 309
+    if ((defined $token)
+      && !$LaTeXML::ALIGN_STATE    # SHOULD count nesting of { }!!! when SCANNED (not digested)
+      && $LaTeXML::READING_ALIGNMENT
+      && (($atoken, $atype, $ahidden) = $self->isColumnEnd($token))) {
+      $self->handleTemplate($LaTeXML::READING_ALIGNMENT, $token, $atype, $ahidden); }
+    else {
+      last; } }
   return $token; }
 
 # Unread tokens are assumed to be not-yet expanded.
@@ -233,7 +304,7 @@ sub unread {
     map { (!defined $_ ? ()
         : (($r = ref $_) eq 'LaTeXML::Core::Token' ? $_
           : ($r eq 'LaTeXML::Core::Tokens' ? @$_
-            : Fatal('misdefined', $r, undef, "Expected a Token, got " . Stringify($_))))) }
+            : Error('misdefined', $r, undef, "Expected a Token, got " . Stringify($_)) || T_OTHER(Stringify($_))))) }
       @tokens);
   return; }
 
@@ -242,39 +313,75 @@ sub unread {
 # `Toplevel' processing, (if $toplevel is true), used at the toplevel processing by Stomach,
 #  will step to the next input stream (Mouth) if one is available,
 # If $commentsok is true, will also pass comments.
+# $toplevel is doing TWO distinct things. When true:
+#  * If a mouth is exhausted, move on to the containing mouth to continue reading
+#  * expand even protected defns, essentially this means expand "for execution"
 sub readXToken {
   my ($self, $toplevel, $commentsok) = @_;
   $toplevel = 1 unless defined $toplevel;
+  my $autoclose      = $toplevel;    # Potentially, these should have distinct controls?
+  my $for_evaluation = $toplevel;
   return shift(@{ $$self{pending_comments} }) if $commentsok && @{ $$self{pending_comments} };
-  my ($token, $cc, $defn);
+  my ($token, $cc, $defn, $atoken, $atype, $ahidden);
   while (1) {
-    $token = shift(@{ $$self{pushback} }) || $$self{mouth}->readToken();
+    # NOTE: CC_SMUGGLE_THE should ONLY appear in pushback!
+    while (($token = shift(@{ $$self{pushback} })) && $CATCODE_HOLD[$cc = $$token[1]]) {
+      if ($cc == CC_COMMENT) {
+        return $token if $commentsok;
+        push(@{ $$self{pending_comments} }, $token); }
+      elsif ($cc == CC_MARKER) {
+        $self->handleMarker($token); } }
+    if (!defined $token) {           # Else read from current mouth
+      while (($token = $$self{mouth}->readToken()) && $CATCODE_HOLD[$cc = $$token[1]]) {
+        if ($cc == CC_COMMENT) {
+          return $token if $commentsok;
+          push(@{ $$self{pending_comments} }, $token); }
+        elsif ($cc == CC_MARKER) {
+          $self->handleMarker($token); } } }
+    ProgressStep() if ($$self{progress}++ % $TOKEN_PROGRESS_QUANTUM) == 0;
     if (!defined $token) {
-      return unless $$self{autoclose} && $toplevel && @{ $$self{mouthstack} };
-      $self->closeMouth; }    # Next input stream.
-    elsif (($cc = $$token[1]) == CC_NOTEXPANDED) {    # NOTE: Inlined ->getCatcode
-          # Should only occur IMMEDIATELY after expanding \noexpand (by readXToken),
-          # so this token should never leak out through an EXTERNAL call to readToken.
-      return $self->readToken; }    # Just return the next token.
-    elsif ($cc == CC_COMMENT) {
-      return $token if $commentsok;
-      push(@{ $$self{pending_comments} }, $token); }    # What to do with comments???
-    elsif ($cc == CC_MARKER) {
-      LaTeXML::Core::Definition::stopProfiling($token, 'expand'); }
-    # Note: special-purpose lookup in State, for efficiency
-    elsif (defined($defn = LaTeXML::Core::State::lookupExpandable($STATE, $token, $toplevel))) {
+      return unless $autoclose && $$self{autoclose} && @{ $$self{mouthstack} };
+      $self->closeMouth; }           # Next input stream.
+        # Handle \noexpand and  smuggled tokens; either expand to $$token[2] or defer till later
+    elsif (my $unexpanded = $$token[2]) {    # Inline get_dont_expand
+      return ($cc != CC_SMUGGLE_THE) || $LaTeXML::SMUGGLE_THE ? $token : $unexpanded; }
+    ## Wow!!!!! See TeX the Program \S 309
+    elsif (!$LaTeXML::ALIGN_STATE    # SHOULD count nesting of { }!!! when SCANNED (not digested)
+      && $LaTeXML::READING_ALIGNMENT
+      && (($atoken, $atype, $ahidden) = $self->isColumnEnd($token))) {
+      $self->handleTemplate($LaTeXML::READING_ALIGNMENT, $token, $atype, $ahidden); }
+    ## Note: use general-purpose lookup, since we may reexamine $defn below
+    elsif ($LaTeXML::Core::State::CATCODE_ACTIVE_OR_CS[$cc]
+      && defined($defn = $STATE->lookupMeaning($token))
+      && ((ref $defn) ne 'LaTeXML::Core::Token')    # an actual definition
+      && $$defn{isExpandable}
+      && ($for_evaluation || !$$defn{isProtected})) { # is this the right logic here? don't expand unless di
       local $LaTeXML::CURRENT_TOKEN = $token;
-      if (my $r = $defn->invoke($self)) {
-        unshift(@{ $$self{pushback} },
-          map { (!defined $_ ? ()
-              : (($r = ref $_) eq 'LaTeXML::Core::Token' ? $_
-                : ($r eq 'LaTeXML::Core::Tokens' ? @$_
-                  : Fatal('misdefined', $r, undef, "Expected a Token, got " . Stringify($_))))) }
-            @{$r}); } }
+      my $r;
+      my @expansion = map { (($r = ref $_) eq 'LaTeXML::Core::Token' ? $_
+          : ($r eq 'LaTeXML::Core::Tokens' ? @$_
+            : Error('misdefined', $r, undef, "Expected a Token, got " . Stringify($_),
+              "in " . ToString($defn)) || T_OTHER(Stringify($_)))) }
+        $defn->invoke($self);
+      next unless @expansion;
+      if ($$LaTeXML::Core::Token::SMUGGLE_THE_COMMANDS{ $$defn{cs}[0] }) {
+        # magic THE_TOKS handling, add to pushback with a single-use noexpand flag only valid
+        # at the exact time the token leaves the pushback.
+        # This is *required to be different* from the noexpand flag, as per the B Book
+        @expansion = map { ($LaTeXML::Core::Token::CATCODE_CAN_SMUGGLE_THE[$$_[1]] ? bless ["SMUGGLE_THE", CC_SMUGGLE_THE, $_], 'LaTeXML::Core::Token' : $_) } @expansion;
+        # PERFORMANCE:
+        #   explicitly flag that we've seen this case, so that higher levels know to
+        #   unset the flag from the entire {pushback}
+        $$self{pushback_has_smuggled_the} = 1; }
+      # add the newly expanded tokens back into the gullet stream, in the ordinary case.
+      unshift(@{ $$self{pushback} }, @expansion); }
+    elsif ($$token[1] == CC_CS && !(defined $defn)) {
+      $STATE->generateErrorStub($self, $token);    # cs SHOULD have defn by now; report early!
+      return $token; }
     else {
-      return $token; }    # just return it
+      return $token; }                             # just return it
   }
-  return; }               # never get here.
+  return; }                                        # never get here.
 
 # Read the next raw line (string);
 # primarily to read from the Mouth, but keep any unread input!
@@ -282,7 +389,7 @@ sub readRawLine {
   my ($self) = @_;
   # If we've got unread tokens, they presumably should come before the Mouth's raw data
   # but we'll convert them back to string.
-  my @tokens  = @{ $$self{pushback} };
+  my @tokens  = map  { ($$_[1] == CC_SMUGGLE_THE ? $$_[2] : $_) } @{ $$self{pushback} };
   my @markers = grep { $_->getCatcode == CC_MARKER } @tokens;
   if (@markers) {    # Whoops, profiling markers!
     @tokens = grep { $_->getCatcode != CC_MARKER } @tokens;    # Remove
@@ -299,6 +406,9 @@ sub readRawLine {
 #**********************************************************************
 # Mid-level readers: checking and matching tokens, strings etc.
 #**********************************************************************
+# General note: TeX uses different tests for Space tokens in different places
+# (possibilities: catcode equality, ->equals, Equals and XEquals)
+
 # The following higher-level parsing methods are built upon readToken & unread.
 sub readNonSpace {
   my ($self) = @_;
@@ -320,10 +430,12 @@ sub skipSpaces {
   unshift(@{ $$self{pushback} }, $tok) if defined $tok;    # Unread
   return; }
 
+# Skip one space
+# if $expanded is true, it acts like <one optional space>, expanding the next token.
 sub skip1Space {
-  my ($self) = @_;
-  my $token = $self->readToken();
-  unshift(@{ $$self{pushback} }, $token) if $token && ($$token[1] != CC_SPACE); # Inline ->getCatcode, unread
+  my ($self, $expanded) = @_;
+  my $token = ($expanded ? $self->readXToken : $self->readToken);
+  unshift(@{ $$self{pushback} }, $token) if $token && !Equals($token, T_SPACE);
   return; }
 
 # <filler> = <optional spaces> | <filler>\relax<optional spaces>
@@ -333,8 +445,8 @@ sub skipFiller {
     my $tok = $self->readNonSpace;
     return unless defined $tok;
     # Should \foo work too (where \let\foo\relax) ??
-    if ($tok->getString ne '\relax') {
-      unshift(@{ $$self{pushback} }, $tok);                                     # Unread
+    if (!$tok->equals(T_CS('\relax'))) {
+      unshift(@{ $$self{pushback} }, $tok);    # Unread
       return; }
   }
   return; }
@@ -342,26 +454,28 @@ sub skipFiller {
 # Read a sequence of tokens balanced in {}
 # assuming the { has already been read.
 # Returns a Tokens list of the balanced sequence, omitting the closing }
-our @balanced_interesting_cc = (
+our @CATCODE_BALANCED_INTERESTING = (
   0, 1, 1, 0,
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 0,
-  0, 0, 1);
+  0, 1, 0, 0);
 
 sub readBalanced {
   my ($self, $expanded) = @_;
+  local $LaTeXML::ALIGN_STATE = 1000000;
   my @tokens = ();
   my ($token, $level) = (undef, 1);
-  my $startloc = $self->getLocator;
+  my $startloc = ($$self{verbosity} > 0) && $self->getLocator;
   # Inlined readToken (we'll keep comments in the result)
   while ($token = ($expanded ? $self->readXToken(0, 1) : $self->readToken())) {
     my $cc = $$token[1];
-    if (!$balanced_interesting_cc[$cc]) {
+    if (!$CATCODE_BALANCED_INTERESTING[$cc]) {
       push(@tokens, $token); }
     elsif ($cc == CC_END) {
       $level--;
-      last unless $level;
+      if (!$level) {
+        last; }
       push(@tokens, $token); }
     elsif ($cc == CC_BEGIN) {
       $level++;
@@ -371,9 +485,9 @@ sub readBalanced {
   if ($level > 0) {
  # TODO: The current implementation has a limitation where if the balancing end is in a different mouth,
  #       it will not be recognized.
+    my $loc_message = $startloc ? ("Started at " . ToString($startloc)) : ("Ended at " . ToString($self->getLocator));
     Error('expected', "}", $self, "Gullet->readBalanced ran out of input in an unbalanced state.",
-      "started at $startloc");
-  }
+      $loc_message); }
   return Tokens(@tokens); }
 
 sub ifNext {
@@ -415,30 +529,59 @@ sub readKeyword {
     my @matched = ();
     my $tok;
     while (@tomatch && defined($tok = $self->readXToken(0)) && push(@matched, $tok)
-      && (uc($tok->getString) eq $tomatch[0])) {
+      && (uc($tok->toString) eq $tomatch[0])) {
       shift(@tomatch); }
-    return $keyword unless @tomatch;    # All matched!!!
+    return $keyword unless @tomatch;             # All matched!!!
     unshift(@{ $$self{pushback} }, @matched);    # Put 'em back and try next!
   }
   return; }
 
 # Return a (balanced) sequence tokens until a match against one of the Tokens in @delims.
-# In list context, also returns the found delimiter.
+# Note that Braces on input hides the contents from matching,
+# so this assumes there wont be braces in $delim!
+# But, see readUntilBrace for that case.
 sub readUntil {
-  my ($self, @delims) = @_;
-  my ($n, $found, @tokens) = (0);
-  while (!defined($found = $self->readMatch(@delims))) {
-    my $token = $self->readToken();    # Copy next token to args
-    return unless defined $token;
-    push(@tokens, $token);
-    $n++;
-    if ($$token[1] == CC_BEGIN) {      # And if it's a BEGIN, copy till balanced END
-      push(@tokens, $self->readBalanced->unlist, T_END); } }
+  my ($self, $delim) = @_;
+  my @tokens = ();
+  my $token;
+  my $nbraces  = 0;
+  my @want     = $delim->unlist;
+  my $ntomatch = scalar(@want);
+  if ($ntomatch == 1) {    # Common, easy case: read till we match a single token
+    my $want = $want[0];
+    #    while(($token = $self->readToken) && !$token->equals($want)){
+    while (($token = shift(@{ $$self{pushback} }) || $$self{mouth}->readToken())
+      && (($$token[1] != CC_SMUGGLE_THE) || ($token = $$token[2]))
+      && !$token->equals($want)) {
+      push(@tokens, $token);
+      if ($$token[1] == CC_BEGIN) {    # And if it's a BEGIN, copy till balanced END
+        $nbraces++;
+        push(@tokens, $self->readBalanced, T_END); } } }
+  else {
+
+    my @ring = ();
+    while (1) {
+      # prefill the required number of tokens
+      while ((scalar(@ring) < $ntomatch) && ($token = $self->readToken)) {
+        if ($$token[1] == CC_BEGIN) {    # read balanced, and refill ring.
+          $nbraces++;
+          push(@tokens, @ring, $token, $self->readBalanced, T_END);    # Copy directly to result
+          @ring = (); }                                                # and retry
+        else {
+          push(@ring, $token); } }
+      my $i;
+      for ($i = 0 ; ($i < $ntomatch) && $ring[$i] && ($ring[$i]->equals($want[$i])) ; $i++) { } # Test match
+      last if $i >= $ntomatch;    # Matched all!
+      last unless $token;
+      push(@tokens, shift(@ring)); } }
+  if (!defined $token) {          # Ran out!
+    $self->unread(@tokens);       # Not more correct, but maybe less confusing?
+    return; }
   # Notice that IFF the arg looks like {balanced}, the outer braces are stripped
   # so that delimited arguments behave more similarly to simple, undelimited arguments.
-  if (($n == 1) && ($tokens[0][1] == CC_BEGIN)) {
+  if (($nbraces == 1) && ($tokens[0][1] == CC_BEGIN) && ($tokens[-1][1] == CC_END)) {
     shift(@tokens); pop(@tokens); }
-  return (wantarray ? (Tokens(@tokens), $found) : Tokens(@tokens)); }
+  return Tokens(@tokens); }
 
 sub readUntilBrace {
   my ($self) = @_;
@@ -451,18 +594,6 @@ sub readUntilBrace {
     push(@tokens, $token); }
   return Tokens(@tokens); }
 
-# Skipping over conditional branches is used heavily when processing raw TeX (eg. tikz).
-# Make this efficient! Note that since we're skipping tokens, we presumably neither
-# need to note profiling end markers, nor preserve comments in the skipped sections?
-sub readNextConditional {
-  my ($self) = @_;
-  my $token;
-  my $type;
-  while ($token = shift(@{ $$self{pushback} }) || $$self{mouth}->readToken()) {
-    if ($type = $STATE->lookupConditional($token)) {
-      return ($token, $type); } }
-  return; }
-
 #**********************************************************************
 # Higher-level readers: Read various types of things from the input:
 #  tokens, non-expandable tokens, args, Numbers, ...
@@ -473,7 +604,7 @@ sub readArg {
   if (!defined $token) {
     return; }
   elsif ($$token[1] == CC_BEGIN) {    # Inline ->getCatcode!
-    return $self->readBalanced; }
+    return $self->readBalanced(0); }
   else {
     return Tokens($token); } }
 
@@ -511,11 +642,13 @@ sub readValue {
 
 sub readRegisterValue {
   my ($self, $type) = @_;
-  my $token = $self->readXToken(0);
+  my $token = $self->readXToken;
   return unless defined $token;
-  my $defn = LaTeXML::Core::State::lookupDefinition($STATE, $token);
+  my $defn = $STATE->lookupDefinition($token);
   if ((defined $defn) && ($defn->isRegister eq $type)) {
-    return $defn->valueOf($defn->readArguments($self)); }
+    local $LaTeXML::CURRENT_TOKEN = $token;
+    my $parms = $$defn{parameters};
+    return $defn->valueOf(($parms ? $parms->readArguments($self) : ())); }
   else {
     unshift(@{ $$self{pushback} }, $token);    # Unread
     return; } }
@@ -528,9 +661,10 @@ sub readTokensValue {
     return; }
   elsif ($$token[1] == CC_BEGIN) {             # Inline ->getCatcode!
     return $self->readBalanced; }
-  elsif (my $defn = LaTeXML::Core::State::lookupDefinition($STATE, $token)) {
+  elsif (my $defn = $STATE->lookupDefinition($token)) {
     if ($defn->isRegister eq 'Tokens') {
-      return $defn->valueOf($defn->readArguments($self)); }
+      my $parms = $$defn{parameters};
+      return $defn->valueOf(($parms ? $parms->readArguments($self) : ())); }
     elsif ($defn->isExpandable) {
       if (my $x = $defn->invoke($self)) {
         $self->unread(@{$x}); }
@@ -542,25 +676,29 @@ sub readTokensValue {
 
 #======================================================================
 # some helpers...
+# Note that <one optional space> is kinda special:
+# The following Token(s) are expanded until an unexpandable token is found;
+# it is discarded if it is a space, but with an Equals() equality test!
 
 # <optional signs> = <optional spaces> | <optional signs><plus or minus><optional spaces>
 # return +1 or -1
 sub readOptionalSigns {
   my ($self) = @_;
   my ($sign, $t) = ("+1", '');
-  while (defined($t = $self->readXToken(0))
-    && (($t->getString eq '+') || ($t->getString eq '-') || ($t->equals(T_SPACE)))) {
+  while (defined($t = $self->readXToken)
+    && (($t->getString eq '+') || ($t->getString eq '-') || Equals($t, T_SPACE))) {
     $sign = -$sign if ($t->getString eq '-'); }
   unshift(@{ $$self{pushback} }, $t) if $t;    # Unread
   return $sign; }
 
+# Read digits (within $range), while expanding and if $skip, skip <one optional space> (expanded!)
 sub readDigits {
   my ($self, $range, $skip) = @_;
   my $string = '';
   my ($token, $digit);
-  while (($token = $self->readXToken(0)) && (($digit = $token->getString) =~ /^[$range]$/)) {
+  while (($token = $self->readXToken) && (($digit = $token->toString) =~ /^[$range]$/)) {
     $string .= $digit; }
-  unshift(@{ $$self{pushback} }, $token) if $token && !($skip && $$token[1] == CC_SPACE); # Inline ->getCatcode, unread
+  unshift(@{ $$self{pushback} }, $token) if $token && !($skip && Equals($token, T_SPACE));    #Inline
   return $string; }
 
 # <factor> = <normal integer> | <decimal constant>
@@ -569,10 +707,10 @@ sub readDigits {
 sub readFactor {
   my ($self) = @_;
   my $string = $self->readDigits('0-9');
-  my $token  = $self->readXToken(0);
+  my $token  = $self->readXToken;
   if ($token && $token->getString =~ /^[\.\,]$/) {
     $string .= '.' . $self->readDigits('0-9');
-    $token = $self->readXToken(0); }
+    $token = $self->readXToken; }
   if (length($string) > 0) {
     unshift(@{ $$self{pushback} }, $token) if $token && $$token[1] != CC_SPACE; # Inline ->getCatcode, unread
     return $string; }
@@ -591,15 +729,14 @@ sub readFactor {
 sub readNumber {
   my ($self) = @_;
   my $s = $self->readOptionalSigns;
-  if (defined(my $n = $self->readNormalInteger)) { return ($s < 0 ? $n->negate : $n); }
+  if    (defined(my $n = $self->readNormalInteger))  { return ($s < 0 ? $n->negate : $n); }
   elsif (defined($n = $self->readInternalDimension)) { return Number($s * $n->valueOf); }
   elsif (defined($n = $self->readInternalGlue))      { return Number($s * $n->valueOf); }
   else {
     my $next = $self->readToken();
     unshift(@{ $$self{pushback} }, $next);    # Unread
     Warn('expected', '<number>', $self, "Missing number, treated as zero",
-      "while processing " . ToString($LaTeXML::CURRENT_TOKEN),
-      "next token is " . ToString($next));
+      "while processing " . ToString($LaTeXML::CURRENT_TOKEN), $self->showUnexpected);
     return Number(0); } }
 
 # <normal integer> = <internal integer> | <integer constant>
@@ -608,19 +745,20 @@ sub readNumber {
 # Return a Number or undef
 sub readNormalInteger {
   my ($self) = @_;
-  my $token = $self->readXToken(0);
+  my $token = $self->readXToken;     # expand more
   if (!defined $token) {
     return; }
-  elsif (($$token[1] == CC_OTHER) && ($token->getString =~ /^[0-9]$/)) {    # Read decimal literal
+  elsif (($$token[1] == CC_OTHER) && ($token->toString =~ /^[0-9]$/)) {    # Read decimal literal
     return Number(int($token->getString . $self->readDigits('0-9', 1))); }
-  elsif ($token->equals(T_OTHER("\'"))) {                                   # Read Octal literal
+  elsif ($token->equals(T_OTHER("'"))) {                                   # Read Octal literal
     return Number(oct($self->readDigits('0-7', 1))); }
-  elsif ($token->equals(T_OTHER("\""))) {                                   # Read Hex literal
+  elsif ($token->equals(T_OTHER("\""))) {                                  # Read Hex literal
     return Number(hex($self->readDigits('0-9A-F', 1))); }
-  elsif ($token->equals(T_OTHER("\`"))) {                                   # Read Charcode
+  elsif ($token->equals(T_OTHER("`"))) {                                   # Read Charcode
     my $next = $self->readToken;
-    my $s    = ($next && $next->getString) || '';
+    my $s    = ($next && $next->toString) || '';
     $s =~ s/^\\//;
+    $self->skip1Space(1);
     return Number(ord($s)); }    # Only a character token!!! NOT expanded!!!!
   else {
     unshift(@{ $$self{pushback} }, $token);    # Unread
@@ -638,10 +776,10 @@ sub readFloat {
   my ($self) = @_;
   my $s      = $self->readOptionalSigns;
   my $string = $self->readDigits('0-9');
-  my $token  = $self->readXToken(0);
+  my $token  = $self->readXToken;
   if ($token && $token->getString =~ /^[\.]$/) {
     $string .= '.' . $self->readDigits('0-9');
-    $token = $self->readXToken(0); }
+    $token = $self->readXToken; }
   my $n;
   if (length($string) > 0) {
     unshift(@{ $$self{pushback} }, $token) if $token && $$token[1] != CC_SPACE; # Inline ->getCatcode, unread
@@ -672,8 +810,8 @@ sub readDimension {
       $unit = 65536; }
     return Dimension($s * $d * $unit); }
   else {
-    Warn('expected', '<number>', $self, "Missing number, treated as zero.",
-      "while processing " . ToString($LaTeXML::CURRENT_TOKEN));
+    Warn('expected', '<number>', $self, "Missing number (Dimension), treated as zero.",
+      "while processing " . ToString($LaTeXML::CURRENT_TOKEN), $self->showUnexpected);
     return Dimension(0); } }
 
 # <unit of measure> = <optional spaces><internal unit>
@@ -686,7 +824,7 @@ sub readDimension {
 sub readUnit {
   my ($self) = @_;
   if (defined(my $u = $self->readKeyword('ex', 'em'))) {
-    $self->skip1Space;
+    $self->skip1Space(1);
     return $STATE->convertUnit($u); }
   elsif (defined($u = $self->readInternalInteger)) {
     return $u->valueOf; }    # These are coerced to number=>sp
@@ -699,7 +837,7 @@ sub readUnit {
     my $units = $STATE->lookupValue('UNITS');
     $u = $self->readKeyword(keys %$units);
     if ($u) {
-      $self->skip1Space;
+      $self->skip1Space(1);
       return $STATE->convertUnit($u); }
     else {
       return; } } }
@@ -735,7 +873,7 @@ sub readMuDimension {
 sub readMuUnit {
   my ($self) = @_;
   if (my $m = $self->readKeyword('mu')) {
-    $self->skip1Space;
+    $self->skip1Space(1);
     return $STATE->convertUnit($m); }
   elsif ($m = $self->readInternalMuGlue) {
     return $m->valueOf; }
@@ -757,8 +895,8 @@ sub readGlue {
   else {
     my $d = $self->readDimension;
     if (!$d) {
-      Warn('expected', '<number>', $self, "Missing number, treated as zero.",
-        "while processing " . ToString($LaTeXML::CURRENT_TOKEN));
+      Warn('expected', '<number>', $self, "Missing number (Glue), treated as zero.",
+        "while processing " . ToString($LaTeXML::CURRENT_TOKEN), $self->showUnexpected);
       return Glue(0); }
     $d = $d->negate if $s < 0;
     my ($r1, $f1, $r2, $f2);
@@ -810,8 +948,8 @@ sub readMuGlue {
   else {
     my $d = $self->readMuDimension;
     if (!$d) {
-      Warn('expected', '<number>', $self, "Missing number, treated as zero.",
-        "while processing " . ToString($LaTeXML::CURRENT_TOKEN));
+      Warn('expected', '<number>', $self, "Missing number (MuGlue), treated as zero.",
+        "while processing " . ToString($LaTeXML::CURRENT_TOKEN), $self->showUnexpected);
       return MuGlue(0); }
     $d = $d->negate if $s < 0;
     my ($r1, $f1, $r2, $f2);
@@ -886,11 +1024,6 @@ Return the L<LaTeXML::Core::Tokens> resulting from expanding all the tokens in C
 This is actually only used in a few circumstances where the arguments to
 an expandable need explicit expansion; usually expansion happens at the right time.
 
-=item C<< @tokens = $gullet->neutralizeTokens(@tokens); >>
-
-Another unusual method: Used for things like \edef and token registers, to
-inhibit further expansion of control sequences and proper spawning of register tokens.
-
 =item C<< $token = $gullet->readToken; >>
 
 Return the next token from the input source, or undef if there is no more input.
@@ -919,14 +1052,16 @@ Read and return the next non-space token from the input after discarding any spa
 
 Skip the next spaces from the input.
 
-=item C<< $gullet->skip1Space; >>
+=item C<< $gullet->skip1Space($expanded); >>
 
 Skip the next token from the input if it is a space.
+If C($expanded> is true, expands ( like C< <one optional space> > ).
 
 =item C<< $tokens = $gullet->readBalanced; >>
 
 Read a sequence of tokens from the input until the balancing '}' (assuming the '{' has
-already been read). Returns a L<LaTeXML::Core::Tokens>.
+already been read). Returns a L<LaTeXML::Core::Tokens>,
+except in an array context, returns the collected tokens and the closing token.
 
 =item C<< $boole = $gullet->ifNext($token); >>
 
